/*
 * <copyright>
 * 
 * TCKFilter.java
 *
 * Licensed Materials - Property of IBM
 * Restricted Materials of IBM
 * 
 * (c) Copyright IBM Corporation 2009.  All Rights Reserved. 
 * Note to U.S. Government Users Restricted Rights:  Use, duplication or  
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp. 
 * 
 * </copyright>
 * 
 */
package com.ibm.sdo.tck.framework;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.Properties;

import org.junit.runner.Description;
import org.junit.runner.manipulation.Filter; 

public class TCKFilter extends Filter {

    static final String FILTER_LIST_TAG = "filterlist";
    static TCKFilter INSTANCE;
    
    private Properties ignoreList;
    
    @Override
    public String describe() 
    {
        return null;
    }

    @Override
    public boolean shouldRun(Description description) 
    {
        // Make a determination whether or not to run 
        // the identified test suite/case.  Return T/F.
        if( ignoreList != null )
        {    
            // This code hacks up the description to provide a 
            // key to be used to identify test cases.  
            String [] keyParts = description.getDisplayName().split("\\(");
            String key = keyParts[0];
            if( keyParts.length > 1 )
            {
                key = keyParts[1]+keyParts[0];
                key = key.replace(')', '.');
            }
            if( ignoreList.containsKey(key) )
                return false;
            else
                return true;
        }
        else
            return true;
    }

    public TCKFilter()
    {
        String listFile = System.getProperty(FILTER_LIST_TAG);
        if( listFile != null )
        {
            Properties list = getIgnoreList();
            
            FileReader is;
            try 
            {
                is = new FileReader(listFile);
                BufferedReader br = new BufferedReader(is);
                String inLine = null;
                while( (inLine=br.readLine()) != null )
                {
                    list.put(inLine.trim(),"");
                }
                
                br.close();
                is.close();
            } 
            catch (FileNotFoundException e) 
            {
                e.printStackTrace();
            } 
            catch (IOException e) 
            {
                e.printStackTrace();
            }
        }
    }
    
    private Properties getIgnoreList()
    {
        if( ignoreList == null )
            ignoreList = new Properties();
        
        return ignoreList;
    }
    
    public void addToIgnoreList(String [] newIgnoreItems )
    {
        if( newIgnoreItems != null )
        {
            for(int i=0; i<newIgnoreItems.length; i++ )
                getIgnoreList().put(newIgnoreItems[i], "");
        }
    }
    
    public static TCKFilter getInstance()
    {
        if( INSTANCE == null )
            INSTANCE = new TCKFilter();
        
        return INSTANCE;
    }
}
