/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.    
 */

package com.ibm.sdo.tck.framework;

import java.lang.annotation.ElementType;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;

import org.junit.internal.runners.InitializationError;
import org.junit.runner.manipulation.NoTestsRemainException;
import org.junit.runners.Suite;

public class TCKSuiteRunner extends Suite {
    
    @Retention(RetentionPolicy.RUNTIME)
    @Target(ElementType.TYPE)
    public @interface TestHelperClass {
            public Class value();
    }
    @Retention(RetentionPolicy.RUNTIME)
    @Target(ElementType.TYPE)    
    public @interface SuiteIgnoreList {
        public String [] value();
    }

    public TCKSuiteRunner( Class klass ) throws InitializationError
    {
        super(klass);

               
        // If available, set the test helper class on TCKTestCase.
        TestHelperClass testHelperClass = (TestHelperClass)klass.getAnnotation(TestHelperClass.class);
        if (testHelperClass != null) {
            TCKTestBase.setHelperClassName(testHelperClass.value().getName());
        }
        
        // Check for test cases to ignore...if so, do so....
        SuiteIgnoreList suiteIgnoreListClass = (SuiteIgnoreList) klass.getAnnotation(SuiteIgnoreList.class);
        if( suiteIgnoreListClass != null ) {
            TCKFilter.getInstance().addToIgnoreList(suiteIgnoreListClass.value());            
        }
        
        try {
            TCKFilter.getInstance().apply(this);
        } 
        catch (NoTestsRemainException e) {
        }
    }
}
