/*
 *  Licensed to the Apache Software Foundation (ASF) under one
 *  or more contributor license agreements.  See the NOTICE file
 *  distributed with this work for additional information
 *  regarding copyright ownership.  The ASF licenses this file
 *  to you under the Apache License, Version 2.0 (the
 *  "License"); you may not use this file except in compliance
 *  with the License.  You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an
 *  "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 *  KIND, either express or implied.  See the License for the
 *  specific language governing permissions and limitations
 *  under the License.
 *
 */
package com.ibm.sdo.tck.framework;

import commonj.sdo.helper.HelperContext;
import commonj.sdo.impl.RuntimeHelper;

/**
 * A superclass for tests classes.  It creates a HelperContext per test case invocation
 * and provides initialization/access to the implementation specific test helper.
 */
public class TCKTestBase {
    
  private static RuntimeHelper testHelper = null; 
  private HelperContext scope = null;
  public static final String TCK_RUNTIMEHELPER_CLASS = "TCK_RUNTIME_HELPER";
  private static String helperClassName = null;
 
  private static void initTestHelper() throws Exception {

      String envHelper = System.getenv(TCK_RUNTIMEHELPER_CLASS) ;
      if( envHelper == null )
          envHelper = System.getProperty(TCK_RUNTIMEHELPER_CLASS);
      if( envHelper != null )
      {
          try 
          {
              testHelper = (RuntimeHelper)Class.forName(envHelper).newInstance();
              helperClassName = envHelper;
              return;
          }
          catch(ClassNotFoundException e)
          {
              System.out.println("The runtime helper specified by the environment variable '" +
                                 TCK_RUNTIMEHELPER_CLASS + "' is not valid (value='" + 
                                 envHelper +"')");
          }
      }
            
      if (helperClassName == null || helperClassName.equals(""))
      {
          System.out.println("Implementation specific test helper was not identified.  This is required prior to executing the TCK." );
          return;
      }
          
      try 
      {
          testHelper = (RuntimeHelper)Class.forName(helperClassName).newInstance();
      }
      catch(ClassNotFoundException e)
      {
          System.out.println("The runtime helper '" + helperClassName + "' is not valid.");
      }
      //System.out.println("Loaded " + helperClassName);
  }

  public static void setHelperClassName(String helper )  {
      TCKTestBase.helperClassName = helper;
  }
  
  public void setUp() throws Exception {
    scope = getTestHelper().createHelperContext();
  }


  public void tearDown() throws Exception {
    scope = null;
  }

  public HelperContext getScope() {
    return scope;
  }

  public static RuntimeHelper getTestHelper() {
    if(testHelper == null) {
      try {
        initTestHelper();
      } catch (Exception e) {
        e.printStackTrace();
      }
    }
    return testHelper;
  }

}
