/*
 * <copyright>
 * 
 * AnnotationsTestBase.java
 *
 * Licensed Materials - Property of IBM
 * Restricted Materials of IBM
 * 
 * (c) Copyright IBM Corporation 2009.  All Rights Reserved. 
 * Note to U.S. Government Users Restricted Rights:  Use, duplication or  
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp. 
 * 
 * </copyright>
 * 
 */
package com.ibm.sdo.tck.testcasebase;

import java.util.List;


import org.junit.Before;
import org.junit.Test;

import com.ibm.sdo.tck.framework.TCKTestBase;
import com.ibm.sdo.tck.framework.annotations.TCKTest;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

import commonj.sdo.DataObject;
import commonj.sdo.Property;
import commonj.sdo.Type;
import commonj.sdo.helper.HelperContext;

@TCKTest()
public abstract class AnnotationsTestBase extends TCKTestBase 
{
    static protected final String TEST_MODEL = "/AnnotationsTest/sdoannotations.xsd";
    static protected final String NS = "http://www.example.com/sdoannotations";
    
    protected HelperContext scope;

    @Before
    public void setUp() throws Exception 
    {
        super.setUp();
        scope = getScope();
    }
    
    @Test
    public void testSDOXMLSequence()
    {
        Type t = scope.getTypeHelper().getType(NS,"seqfoo1");
        assertTrue(t.isSequenced());
    }
    
    @Test
    public void testSDOXMLAliasName()
    {
        Type t = scope.getTypeHelper().getType(NS,"aliasfoo1");
        List l  = t.getAliasNames();
        assertEquals(1,l.size());
        Property p1 = t.getProperty("aliasbar1");
        Property p2 = t.getProperty("aliasbar2");
        l = p1.getAliasNames();
        assertEquals(3,l.size());
        l = p2.getAliasNames();
        if( l != null )
        {
            // If l == null, that is ok, otherwise we'll check
            // the size.
            assertEquals(0,l.size());
        }
        
        DataObject obj = scope.getDataFactory().create(t);
        obj.setString(p1, "value1");
        obj.setString("aliasbar7", "value2");
        assertEquals("value2",obj.getString("aliasbar8"));
    }
        
    
    @Test
    public void testSDOXMLReadOnly()
    {
        Type t = scope.getTypeHelper().getType(NS,"rofoo1");
        Property p1 = t.getProperty("robar1");
        Property p2 = t.getProperty("robar2");
        Property p3 = t.getProperty("robar3");
        assertFalse(p1.isReadOnly());
        assertTrue(p2.isReadOnly());
        assertFalse(p3.isReadOnly());
        
        DataObject obj = scope.getDataFactory().create(t);
       
        try
        {
            obj.setString(p1, "value1");
            obj.setString(p2, "value2");
        }
        catch(Exception e)
        {
            // it is expected that we may fall into here when
            // p2 is being set.  
        }
        
        assertEquals("value1",obj.getString(p1));
        assertEquals(p2.getDefault(),obj.getString(p2));
    }

    
    @Test
    public void testSDOXMLMany()
    {
        Type t = scope.getTypeHelper().getType(NS,"manyfoo1");
        Property p1 = t.getProperty("manybar1");
        Property p2 = t.getProperty("manybar2");
        Property p3 = t.getProperty("manybar3");
        assertFalse(p1.isMany());
        assertTrue(p2.isMany());
        assertFalse(p3.isMany());
    }
    
    @Test
    public void testSDOXMLPropertyTypeOppositeProperty()
    {
        Type t1 = scope.getTypeHelper().getType(NS,"reffoo1");
        Type t2 = scope.getTypeHelper().getType(NS,"reffoo2");
        Property pref1 = t1.getProperty("refbar1");
        Property pref2 = t2.getProperty("refbar2");

        Property o1 = pref1.getOpposite();
        Property o2 = pref2.getOpposite();
        
        assertEquals(pref1,o2);
        assertEquals(pref2,o1);
        assertEquals(t1,pref2.getType());
        assertEquals(t2,pref1.getType());
    }

}
