/*
 *  Licensed to the Apache Software Foundation (ASF) under one
 *  or more contributor license agreements.  See the NOTICE file
 *  distributed with this work for additional information
 *  regarding copyright ownership.  The ASF licenses this file
 *  to you under the Apache License, Version 2.0 (the
 *  "License"); you may not use this file except in compliance
 *  with the License.  You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an
 *  "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 *  KIND, either express or implied.  See the License for the
 *  specific language governing permissions and limitations
 *  under the License.
 *  
 *  $Rev: 538928 $  $Date: 2008/02/06 21:04:48 $
 */
package com.ibm.sdo.tck.testcasebase;

import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;


import org.junit.Test;

import com.ibm.sdo.tck.framework.TCKTestBase;
import com.ibm.sdo.tck.framework.annotations.Operation;
import com.ibm.sdo.tck.framework.annotations.TCKTest;
import com.ibm.sdo.tck.util.EqualityTest;


import commonj.sdo.DataObject;

/**
 * TestCase tests shallow and deep copy functions of CopyHelper. 
 * Can be used to test dynamic, mixed and static
 * DataObjects
 */
@TCKTest(testFor={
    @Operation(className="CopyHelper",methodName="copy",signature="DataObject"),
    @Operation(className="CopyHelper",methodName="copyShallow",signature="DataObject"),
    @Operation(className="DataObject", methodName="unset", signature="String"),
    @Operation(className="HelperContext", methodName="getCopyHelper", signature="")
    })
public abstract class CopyHelperTestBase extends TCKTestBase 
{
	protected DataObject testDO;
	
    /**
     * Performs a shallow copy and verifies that shallow equality is true
     * between the source and output for the copy.
     */
    @Test
    public void testShallowCopyShallowEquality() {
        assertNotNull("data object not created correctly", testDO);
        DataObject shallowCopy = getScope().getCopyHelper().copyShallow(testDO);
        
        EqualityTest et = new EqualityTest(testDO,shallowCopy,false);
        assertTrue("Shallow copy did not result in shallow equality: " + et.getFailure(), et.getResult() );
    }

    /**
     * Performs a shallow copy and verifies that deep equality is false
     * (assuming passed DO includes a containment reference) between the source
     * and output for the copy.
     */
    @Test
    public void testShallowCopyDeepEquality() {
        assertNotNull("data object not created correctly", testDO);
        DataObject shallowCopy = getScope().getCopyHelper().copyShallow(testDO);

        EqualityTest et = new EqualityTest(testDO,shallowCopy,true);
        assertFalse("Shallow copy should not result in deep equality for this Type: "+et.getFailure(), 
                     et.getResult());
    }

    /**
     * Performs a deep copy and verifies that shallow equality exists between
     * the source and output from the copy.
     */
    @Test
    public void testDeepCopyShallowEquality() {
        assertNotNull("data object not created correctly", testDO);
        DataObject deepCopy = getScope().getCopyHelper().copy(testDO);

        EqualityTest et = new EqualityTest(testDO,deepCopy,false);
        assertTrue("Deep copy did not result in shallow equality: " + et.getFailure(), et.getResult() );
    }

    /**
     * Performs a deep copy and verifies that deep equality exists between the
     * source and output from the copy.
     */
    @Test
    public void testDeepCopyDeepEquality() {
        assertNotNull("data object not created correctly", testDO);
        DataObject deepCopy = getScope().getCopyHelper().copy(testDO);

        EqualityTest et = new EqualityTest(testDO,deepCopy,true);
        assertTrue("Deep copy did not result in deep equality: "+et.getFailure(), et.getResult());
    }
}
