/*
 * <copyright>
 * 
 * PathTestBase.java
 *
 * Licensed Materials - Property of IBM
 * Restricted Materials of IBM
 * 
 * (c) Copyright IBM Corporation 2009.  All Rights Reserved. 
 * Note to U.S. Government Users Restricted Rights:  Use, duplication or  
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp. 
 * 
 * </copyright>
 * 
 */package com.ibm.sdo.tck.testcasebase;

import java.io.IOException;
import java.io.InputStream;
import java.net.URL;


import junit.framework.Assert;

import org.junit.Before;

import com.ibm.sdo.tck.framework.TCKTestBase;

import commonj.sdo.DataObject;
import commonj.sdo.helper.DataFactory;
import commonj.sdo.helper.HelperContext;
import commonj.sdo.helper.XMLHelper;
import commonj.sdo.helper.XSDHelper;

public class PathTestBase extends TCKTestBase {
    protected XMLHelper xmlHelper;
	protected XSDHelper xsdHelper;
	protected DataFactory dataFactory;

	protected String RES_PATH_BOOKSTOREXSD = "/PathTest/bookstore.xsd";
	protected String RES_PATH_PERSONXSD = "/PathTest/person.xsd";

	protected String RES_PATH_BOOKSTOREXML = "/PathTest/bookstore.xml";
    protected String RES_PATH_PERSONXML = "/PathTest/person.xml";

	protected final String xsdString = "<xsd:schema targetNamespace=\"http://www.example.com/simple\" "
           + "xmlns:xsd=\"http://www.w3.org/2001/XMLSchema\" "
           + "xmlns:simple=\"http://www.example.com/simple\"> "
           + "<xsd:element name=\"stockQuote\" type=\"simple:Quote\"/> "
           + "<xsd:element name=\"stockQuote2\" type=\"simple:Quote\"/> "
           + "<xsd:complexType name=\"Quote\"> "
           + "<xsd:sequence> "
           + "<xsd:element name=\"symbol\" type=\"xsd:string\"/> "
           + "<xsd:element name=\"policy\" type=\"xsd:QName\"/> "
           + "</xsd:sequence> " + "</xsd:complexType> " + "</xsd:schema>";

	protected final String xmlString = "<?xml version=\"1.0\" encoding=\"ASCII\"?> "
           + "<simple:stockQuote xmlns:simple=\"http://www.example.com/simple\"> "
           + "<symbol>fbnt</symbol> "
           + "<policy>simple:stockQuote</policy> "
           + "</simple:stockQuote>";

	protected void bookstoreTest_set_bracketIndex(String path) throws IOException {
		URL url = getClass().getResource(RES_PATH_BOOKSTOREXML);
		InputStream is = url.openStream();
		DataObject root = xmlHelper.load(is).getRootObject();

		Assert.assertTrue(root.isSet(path));
		Assert.assertEquals("Per Bothner", root.getString(path));
		root.unset(path);
		// Make sure the next author took the new index position
		Assert.assertEquals("Kurt Cagle", root.getString(path));
		root.set(path, "Tom Clancy");
		Assert.assertEquals("Tom Clancy", root.getString(path));
	}


	@Before
	public void setUp() throws Exception {
	    super.setUp();
		HelperContext hc = getScope();
		xsdHelper = hc.getXSDHelper();
		xmlHelper = hc.getXMLHelper();
		dataFactory = hc.getDataFactory();

		URL url = getClass().getResource(RES_PATH_BOOKSTOREXSD);
		InputStream is = url.openStream();
		xsdHelper.define(is, url.toString());
		is.close();

	}
}
