/*
 *  Licensed to the Apache Software Foundation (ASF) under one
 *  or more contributor license agreements.  See the NOTICE file
 *  distributed with this work for additional information
 *  regarding copyright ownership.  The ASF licenses this file
 *  to you under the Apache License, Version 2.0 (the
 *  "License"); you may not use this file except in compliance
 *  with the License.  You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an
 *  "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 *  KIND, either express or implied.  See the License for the
 *  specific language governing permissions and limitations
 *  under the License.
 *  
 *  $Rev: 536535 $  $Date: 2008/01/15 11:17:51 $
 */
package com.ibm.sdo.tck.testcasebase;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;

import java.util.List;


import org.junit.Test;

import com.ibm.sdo.tck.framework.TCKTestBase;
import com.ibm.sdo.tck.framework.annotations.Operation;
import com.ibm.sdo.tck.framework.annotations.TCKTest;
import com.ibm.sdo.tck.framework.annotations.Operation.ErrorOutcome;
import com.ibm.sdo.tck.util.TCKUtil;


import commonj.sdo.DataObject;
import commonj.sdo.Property;
import commonj.sdo.Sequence;

@TCKTest(testFor={@Operation(className="Sequence", methodName="addText", signature="String"),
                  @Operation(className="Sequence", methodName="addText", signature="int,String"),     
                  @Operation(className="Sequence", methodName="add", signature="int,int,Object"),
                  @Operation(className="Sequence", methodName="add", signature="int,int,Object", precondition="Invalid Object", expectedException=ErrorOutcome.EXCEPTION),
                  @Operation(className="Sequence", methodName="add", signature="int,int,Object", precondition="Invalid Property Index", expectedException=ErrorOutcome.EXCEPTION),
                  @Operation(className="Sequence", methodName="add", signature="int,int,Object", precondition="Invalid Sequence Index", expectedException=ErrorOutcome.EXCEPTION),
                  @Operation(className="Sequence", methodName="add", signature="int,Object"),
                  @Operation(className="Sequence", methodName="add", signature="int,Object", precondition="Invalid Sequence Index", expectedException=ErrorOutcome.EXCEPTION),
                  @Operation(className="Sequence", methodName="add", signature="int,Object", precondition="Invalid Object", expectedException=ErrorOutcome.EXCEPTION),
                  @Operation(className="Sequence", methodName="add", signature="int,String"),
                  @Operation(className="Sequence", methodName="add", signature="int,Property,Object", precondition="Invalid Object", expectedException=ErrorOutcome.EXCEPTION),
                  @Operation(className="Sequence", methodName="add", signature="int,Property,Object", precondition="Invalid Property", expectedException=ErrorOutcome.EXCEPTION),
                  @Operation(className="Sequence", methodName="add", signature="int,Property,Object", precondition="Invalid Sequence Index", expectedException=ErrorOutcome.EXCEPTION),
                  @Operation(className="Sequence", methodName="add", signature="int,String,Object"),
                  @Operation(className="Sequence", methodName="add", signature="int,String,Object", precondition="Invalid Object", expectedException=ErrorOutcome.EXCEPTION),
                  @Operation(className="Sequence", methodName="add", signature="int,String,Object", precondition="Invalid Sequence Index", expectedException=ErrorOutcome.EXCEPTION),
                  @Operation(className="Sequence", methodName="add", signature="int,String,Object", precondition="Invalid String", expectedException=ErrorOutcome.EXCEPTION),
                  @Operation(className="Sequence", methodName="add", signature="Property,Object"),                 
                  @Operation(className="Sequence", methodName="add", signature="Property,Object", precondition="Invalid Object", expectedException=ErrorOutcome.EXCEPTION),                 
                  @Operation(className="Sequence", methodName="add", signature="Property,Object", precondition="Invalid Property", expectedException=ErrorOutcome.EXCEPTION),                 
                  @Operation(className="Sequence", methodName="add", signature="String,Object"),
                  @Operation(className="Sequence", methodName="add", signature="String,Object", precondition="Invalid Object", expectedException=ErrorOutcome.EXCEPTION),
                  @Operation(className="Sequence", methodName="add", signature="String,Object", precondition="Invalid String", expectedException=ErrorOutcome.EXCEPTION),
                  @Operation(className="Sequence", methodName = "getProperty", signature = "int"),
                  @Operation(className="Sequence", methodName = "getValue", signature = "int"),
                  @Operation(className="Sequence", methodName="move", signature="int,int"),
                  @Operation(className="Sequence", methodName="remove", signature="int"),
                  @Operation(className="Sequence", methodName = "setValue", signature = "int,Object"),
                  @Operation(className="Type", methodName="isSequenced", signature=""),
                  @Operation(className="DataObject", methodName = "getSequence", signature = "")})
public abstract class SequenceTestBase extends TCKTestBase {
    protected   DataObject sequenceDO;
    protected   DataObject testDO;

    /**
     * getIndex is a private method used by the SequenceConsistencyBase class to accept as
     * input a Property and return the index of that Property within the
     * 'Sequenced' Type
     */
    private int getIndex(Property property) {
        List properties = sequenceDO.getType().getProperties();
        int propertyIndex = -1;
        int i = 0;

        while (i < properties.size() && propertyIndex < 0) {
            if (((Property)properties.get(i)).equals(property))
                propertyIndex = i;
            i++;
        }

        return propertyIndex;
    }

    /**
     * populateSequence is called before each test. It provides a set of
     * consistent data for the tests to work with. The expected outcome of
     * populateSequence is a Sequence as follows: {<Letters, "A">, <Letters,
     * "B">, <Numbers, 5>, <Letters, "C">, <Numbers, 16>}
     * @throws Exception 
     */
    protected void populateSequence() throws Exception {
        sequenceDO = TCKUtil.createTestDataObject(getScope(), TCKUtil.SEQ_TYPE);

        if (sequenceDO.getType().isSequenced()) {
            Sequence sequence = sequenceDO.getSequence();

            // Add elements to the Sequence by updating the DataObject

            /*
            List letterList = sequenceDO.getList("Letters");
            List numberList = sequenceDO.getList("Numbers");

            letterList.add("A");
            letterList.add("B");
            numberList.add(Integer.valueOf(5));
            letterList.add("C");
            numberList.add(Integer.valueOf(16));          
            */
            sequence.add(0, "Letters", "A");
            sequence.add(1, "Letters", "B");
            sequence.add(2, "Numbers", Integer.valueOf(5));
            sequence.add(3, "Letters", "C");
            sequence.add(4, "Numbers", Integer.valueOf(16));
        }
    }

    /**
     * Verify that Sequence.size() returns the expected value.
     */
    @Test
    public void testSize() {
        assertTrue("The created Type 'Sequenced' has Type.isSequenced() == false.  The test cases may not proceed.",
                   sequenceDO.getType().isSequenced());

        Sequence sequence = sequenceDO.getSequence();

        assertEquals("Sequence.size() returned an unexpected value.", sequence.size(), 5);
    }

    /**
     * Verify that Sequence.getProperty() returns the expected values.
     */
    @Test
    public void testGetProperty() {
        assertTrue("The created Type 'Sequenced' has Type.isSequenced() == false.  The test cases may not proceed.",
                   sequenceDO.getType().isSequenced());

        Sequence sequence = sequenceDO.getSequence();

        assertEquals("Sequence.getProperty() returned an unexpected Property.",
                     sequenceDO.getInstanceProperty("Letters"),
                     sequence.getProperty(1));
        assertEquals("Sequence.getProperty() returned an unexpected Property.",
                     sequenceDO.getInstanceProperty("Numbers"),
                     sequence.getProperty(4));
    }

    /**
     * Verify that Sequence.getValue() returns the expected values.
     */
    @Test
    public void testGetValue() {
        assertTrue("The created Type 'Sequenced' has Type.isSequenced() == false.  The test cases may not proceed.",
                   sequenceDO.getType().isSequenced());

        Sequence sequence = sequenceDO.getSequence();

        assertEquals("Sequence.getValue() returned an unexpected value.", "A", sequence.getValue(0));
        assertEquals("Sequence.getValue() returned an unexpected value.", 5, sequence.getValue(2));
    }

    /**
     * Use Sequence.setValue() to modify the Sequence, then verify the outcome
     */
    @Test
    public void testSetValue() {
        assertTrue("The created Type 'Sequenced' has Type.isSequenced() == false.  The test cases may not proceed.",
                   sequenceDO.getType().isSequenced());

        Sequence sequence = sequenceDO.getSequence();

        sequence.setValue(1, "F");

        // {<Letters, "A">, <Letters, "F">, <Numbers, 5>, <Letters, "C">,
        // <Numbers, 16>}

        assertEquals("Sequence.setValue() did not have the intended effect.", sequence.getValue(1), "F");

        List letters = sequenceDO.getList("Letters");
        assertEquals("Sequence.setValue() had an unexpected effect on the size of the List in the underlying DataObject.",
                     3,
                     letters.size());
        assertEquals("Sequence.setValue() did not have the expected effect on the underlying DataObject.",
                     "F",
                     (String)letters.get(1));
    }
    
    /**
     * Test to ensure that once a property within a sequenced type
     * retains an existing position in the sequence following an update.
     */
    @Test
    public void testSetValuePosition()
    {
        Sequence sequence = sequenceDO.getSequence();
        assertEquals(sequence.size(), 5);
        assertEquals("A", sequence.getValue(0));
      
        sequenceDO.set("Letters[1]", "X");
      
        assertEquals(sequence.size(), 5);
        assertEquals("X", sequence.getValue(0));
    }


    /**
     * Use Sequence.remove() to modify the Sequence, then verify the outcome
     */
    @Test
    public void testRemove() {
        assertTrue("The created Type 'Sequenced' has Type.isSequenced() == false.  The test cases may not proceed.",
                   sequenceDO.getType().isSequenced());

        Sequence sequence = sequenceDO.getSequence();

        sequence.remove(1);

        // {<Letters, "A">, <Numbers, 5>, <Letters, "C">, <Numbers, 16>}

        assertEquals("Sequence.remove() did not decrement the size of the Sequence.", 4, sequence.size());
        assertEquals("Sequence.remove() did not shift the elements as expected.", 5, sequence.getValue(1));

        List letters = sequenceDO.getList("Letters");
        assertEquals("Sequence.remove() did not have the expected effect on the size of the List in the underlying DataObject.",
                     2,
                     letters.size());
        assertEquals("Sequence.remove() did not have the expected effect on the underlying DataObject.",
                     "C",
                     (String)letters.get(1));
    }

    /**
     * Use Sequence.addText(String) to modify the Sequence, then verify the
     * outcome
     */
    @Test
    public void testAddText() {
        assertTrue("The created Type 'Sequenced' has Type.isSequenced() == false.  The test cases may not proceed.",
                   sequenceDO.getType().isSequenced());

        Sequence sequence = sequenceDO.getSequence();

        String addString = new String("Structured text at the end.");
        sequence.addText(addString);

        // {<Letters, "A">, <Letters, "B">, <Numbers, 5>, <Letters, "C">,
        // <Numbers, 16>, "Structured text at the end."}

        assertEquals("Sequence.add(String) did not increment the size of the Sequence.", 6, sequence.size());
        assertEquals("Sequence.add(String) did not place the correct value at the correct index.",
                     addString,
                     sequence.getValue(5));
        assertNull("Sequence.add(String) should result in a null Property in the final index.", sequence.getProperty(5));
    }

    /**
     * Use Sequence.addText(int, String) to modify the Sequence, then verify the
     * outcome
     */
    @Test
    public void testAddTextWithIndex() {
        assertTrue("The created Type 'Sequenced' has Type.isSequenced() == false.  The test cases may not proceed.",
                   sequenceDO.getType().isSequenced());

        Sequence sequence = sequenceDO.getSequence();

        String addString = new String("Structured text in the middle.");
        sequence.addText(2, addString);

        // {<Letters, "A">, <Letters, "B">, "Structured text in the middle.",
        // <Numbers, 5>, <Letters, "C">, <Numbers, 16>}

        assertEquals("Sequence.addText(int, String) did not increment the size of the Sequence.", 6, sequence
            .size());
        assertEquals("Sequence.addText(int, String) did not place the correct value at the correct index.",
                     addString,
                     sequence.getValue(2));
        assertNull("Sequence.addText(int, String) should result in a null Property in the specified index.",
                   sequence.getProperty(2));
    }

    /**
     * Use Sequence.move() to modify the Sequence, then verify the outcome
     */
    @Test
    public void testMove() {
        assertTrue("The created Type 'Sequenced' has Type.isSequenced() == false.  The test cases may not proceed.",
                   sequenceDO.getType().isSequenced());

        Sequence sequence = sequenceDO.getSequence();

        sequence.move(1, 3);

        // {<Letters, "A">, <Letters, "C">, <Letters, "B">, <Numbers, 5>,
        // <Numbers, 16>}

        assertEquals("Sequence.move() had an unexpected effect on the size of the Sequence.", 5, sequence.size());
        assertEquals("Sequence.move() did not place the expected value at the expected location.", "C", sequence
            .getValue(1));
        assertEquals("Sequence.move() had an unexpected effect on the index following the move-to index.",
                     "B",
                     sequence.getValue(2));

        List letters = sequenceDO.getList("Letters");
        assertEquals("Sequence.remove() did not have the expected effect on the size of the List in the underlying DataObject.",
                     3,
                     letters.size());
        assertEquals("Sequence.remove() did not have the expected effect on the underlying DataObject.",
                     "A",
                     (String)letters.get(0));
        assertEquals("Sequence.remove() did not have the expected effect on the underlying DataObject.",
                     "C",
                     (String)letters.get(1));
        assertEquals("Sequence.remove() did not have the expected effect on the underlying DataObject.",
                     "B",
                     (String)letters.get(2));
    }

    /**
     * Use Sequence.add(Property, Object) to modify the Sequence, then verify
     * the outcome
     */
    @Test
    public void testAddPropertyObject() {
        assertTrue("The created Type 'Sequenced' has Type.isSequenced() == false.  The test cases may not proceed.",
                   sequenceDO.getType().isSequenced());

        Sequence sequence = sequenceDO.getSequence();
        Property numberProp = sequenceDO.getInstanceProperty("Numbers");

        sequence.add(numberProp, Integer.valueOf(8));

        // {<Letters, "A">, <Letters, "B">, <Numbers, 5>, <Letters, "C">,
        // <Numbers, 16>, <Numbers, 8>}

        assertEquals("Sequence.add(Property, Object) did not increment the size of the Sequence.", 6, sequence
            .size());
        assertEquals("Sequence.add(Property, Object) did not place the expected value at the expected index.",
                     8,
                     sequence.getValue(5));
        assertEquals("Sequence.add(Property, Object) did not place the expected Property at the expected index.",
                     numberProp,
                     sequence.getProperty(5));

        List numbers = sequenceDO.getList("Numbers");

        assertEquals("Sequence.add(Property, Object) did not have the expected effect on the size of the List in the underlying DataObject.",
                     3,
                     numbers.size());
        assertEquals("Sequence.add(Property, Object) did not have the expected effect on the underlying DataObject.",
                     5,
                     numbers.get(0));
        assertEquals("Sequence.add(Property, Object) did not have the expected effect on the underlying DataObject.",
                     16,
                     numbers.get(1));
        assertEquals("Sequence.add(Property, Object) did not have the expected effect on the underlying DataObject.",
                     8,
                     numbers.get(2));
    }

    /**
     * Use Sequence.add(String, Object) to modify the Sequence, then verify the
     * outcome
     */
    @Test
    public void testAddStringObject() {
        assertTrue("The created Type 'Sequenced' has Type.isSequenced() == false.  The test cases may not proceed.",
                   sequenceDO.getType().isSequenced());

        Sequence sequence = sequenceDO.getSequence();

        sequence.add("Letters", "K");

        // {<Letters, "A">, <Letters, "B">, <Numbers, 5>, <Letters, "C">,
        // <Numbers, 16>, <Letters, "K">}

        assertEquals("Sequence.add(String, Object) did not increment the size of the Sequence.", 6, sequence
            .size());
        assertEquals("Sequence.add(String, Object) did not place the expected value at the expected index.",
                     "K",
                     sequence.getValue(5));
        assertEquals("Sequence.add(String, Object) did not place the expected Property at the expected index.",
                     sequenceDO.getInstanceProperty("Letters"),
                     sequence.getProperty(5));

        List letters = sequenceDO.getList("Letters");
        assertEquals("Sequence.add(String, Object) did not have the expected effect on the size of the List in the underlying DataObject.",
                     4,
                     letters.size());
        assertEquals("Sequence.add(String, Object) did not have the expected effect on the underlying DataObject.",
                     "A",
                     (String)letters.get(0));
        assertEquals("Sequence.add(String, Object) did not have the expected effect on the underlying DataObject.",
                     "B",
                     (String)letters.get(1));
        assertEquals("Sequence.add(String, Object) did not have the expected effect on the underlying DataObject.",
                     "C",
                     (String)letters.get(2));
        assertEquals("Sequence.add(String, Object) did not have the expected effect on the underlying DataObject.",
                     "K",
                     (String)letters.get(3));
    }

    /**
     * Use Sequence.add(int, Object) to modify the Sequence, then verify the
     * outcome
     */
    @Test
    public void testAddIntObject() {
        assertTrue("The created Type 'Sequenced' has Type.isSequenced() == false.  The test cases may not proceed.",
                   sequenceDO.getType().isSequenced());

        Sequence sequence = sequenceDO.getSequence();

        sequence.add(getIndex(sequenceDO.getInstanceProperty("Numbers")), Integer.valueOf(10));

        // {<Letters, "A">, <Letters, "B">, <Numbers, 5>, <Letters, "C">,
        // <Numbers, 16>, <Numbers, 10>}

        assertEquals("Sequence.add(Property, Object) did not increment the size of the Sequence.", 6, sequence
            .size());
        assertEquals("Sequence.add(Property, Object) did not place the expected value at the expected index.",
                     10,
                     sequence.getValue(5));
        assertEquals("Sequence.add(Property, Object) did not place the expected Property at the expected index.",
                     sequenceDO.getInstanceProperty("Numbers"),
                     sequence.getProperty(5));

        List numbers = sequenceDO.getList("Numbers");

        assertEquals("Sequence.add(Property, Object) did not have the expected effect on the size of the List in the underlying DataObject.",
                     3,
                     numbers.size());
        assertEquals("Sequence.add(Property, Object) did not have the expected effect on the underlying DataObject.",
                     5,
                     numbers.get(0));
        assertEquals("Sequence.add(Property, Object) did not have the expected effect on the underlying DataObject.",
                     16,
                     numbers.get(1));
        assertEquals("Sequence.add(Property, Object) did not have the expected effect on the underlying DataObject.",
                     10,
                     numbers.get(2));
    }

    /**
     * Use Sequence.add(int, String, Object) to modify the Sequence, then verify
     * the outcome
     */
    @Test
    public void testAddIntStringObject() {
        assertTrue("The created Type 'Sequenced' has Type.isSequenced() == false.  The test cases may not proceed.",
                   sequenceDO.getType().isSequenced());

        Sequence sequence = sequenceDO.getSequence();

        int sequenceIndex = 0;

        sequence.add(sequenceIndex, "Numbers", Integer.valueOf(10));

        // {<Numbers, 10>, <Letters, "A">, <Letters, "B">, <Numbers, 5>,
        // <Letters, "C">, <Numbers, 16>}

        assertEquals("Sequence.add(int, String, Object) did not increment the size of the Sequence.", 6, sequence
            .size());
        assertEquals("Sequence.add(int, String, Object) did not place the expected value at the expected index.",
                     10,
                     sequence.getValue(sequenceIndex));
        assertEquals("Sequence.add(int, String, Object) did not place the expected Property at the expected index.",
                     sequenceDO.getInstanceProperty("Numbers"),
                     sequence.getProperty(sequenceIndex));

        List numbers = sequenceDO.getList("Numbers");
        assertEquals("Sequence.add(int, String, Object) did not have the expected effect on the size of the List in the underlying DataObject.",
                     3,
                     numbers.size());
        assertEquals("Sequence.add(int, String, Object) did not have the expected effect on the underlying DataObject.",
                     10,
                     numbers.get(0));
        assertEquals("Sequence.add(int, String, Object) did not have the expected effect on the underlying DataObject.",
                     5,
                     numbers.get(1));
        assertEquals("Sequence.add(int, String, Object) did not have the expected effect on the underlying DataObject.",
                     16,
                     numbers.get(2));
    }

    /**
     * Use Sequence.add(int, Property, Object) to modify the Sequence, then
     * verify the outcome
     */
    @Test
    public void testAddIntPropertyObject() {
        assertTrue("The created Type 'Sequenced' has Type.isSequenced() == false.  The test cases may not proceed.",
                   sequenceDO.getType().isSequenced());

        Sequence sequence = sequenceDO.getSequence();

        int sequenceIndex = 3;
        sequence.add(sequenceIndex, "Letters", "K");

        // {<Letters, "A">, <Letters, "B">, <Numbers, 5>, <Letters, "K">,
        // <Letters, "C">, <Numbers, 16>}

        assertEquals("Sequence.add(int, Property, Object) did not increment the size of the Sequence.",
                     6,
                     sequence.size());
        assertEquals("Sequence.add(int, Property, Object) did not place the expected value at the expected index.",
                     "K",
                     sequence.getValue(sequenceIndex));
        assertEquals("Sequence.add(int, Property, Object) did not place the expected Property at the expected index.",
                     sequenceDO.getInstanceProperty("Letters"),
                     sequence.getProperty(sequenceIndex));

        List letters = sequenceDO.getList("Letters");
        assertEquals("Sequence.add(int, Property, Object) did not have the expected effect on the size of the List in the underlying DataObject.",
                     4,
                     letters.size());
        assertEquals("Sequence.add(int, Property, Object) did not have the expected effect on the underlying DataObject.",
                     "A",
                     (String)letters.get(0));
        assertEquals("Sequence.add(int, Property, Object) did not have the expected effect on the underlying DataObject.",
                     "B",
                     (String)letters.get(1));
        assertEquals("Sequence.add(int, Property, Object) did not have the expected effect on the underlying DataObject.",
                     "K",
                     (String)letters.get(2));
        assertEquals("Sequence.add(int, Property, Object) did not have the expected effect on the underlying DataObject.",
                     "C",
                     (String)letters.get(3));
    }

    /**
     * Use Sequence.add(int, int, Object) to modify the Sequence, then verify
     * the outcome
     */
    @Test
    public void testAddIntIntObject() {
        assertTrue("The created Type 'Sequenced' has Type.isSequenced() == false.  The test cases may not proceed.",
                   sequenceDO.getType().isSequenced());

        Sequence sequence = sequenceDO.getSequence();

        int sequenceIndex = 5;

        sequence.add(sequenceIndex, "Numbers", Integer.valueOf(10));

        // {<Letters, "A">, <Letters, "B">, <Numbers, 5>, <Letters, "C">,
        // <Numbers, 16>, <Numbers, 10>}

        assertEquals("Sequence.add(int, String, Object) did not increment the size of the Sequence.", 6, sequence
            .size());
        assertEquals("Sequence.add(int, String, Object) did not place the expected value at the expected index.",
                     10,
                     sequence.getValue(sequenceIndex));
        assertEquals("Sequence.add(int, String, Object) did not place the expected Property at the expected index.",
                     sequenceDO.getInstanceProperty("Numbers"),
                     sequence.getProperty(sequenceIndex));

        List numbers = sequenceDO.getList("Numbers");
        assertEquals("Sequence.add(int, String, Object) did not have the expected effect on the size of the List in the underlying DataObject.",
                     3,
                     numbers.size());
        assertEquals("Sequence.add(int, String, Object) did not have the expected effect on the underlying DataObject.",
                     5,
                     numbers.get(0));
        assertEquals("Sequence.add(int, String, Object) did not have the expected effect on the underlying DataObject.",
                     16,
                     numbers.get(1));
        assertEquals("Sequence.add(int, String, Object) did not have the expected effect on the underlying DataObject.",
                     10,
                     numbers.get(2));
    }
}
