/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.    
 */

package com.ibm.sdo.tck.tests.core;

import static org.junit.Assert.*;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertNotSame;
import static org.junit.Assert.assertSame;

import java.io.InputStream;
import java.net.URL;


import org.junit.Before;
import org.junit.Test;

import com.ibm.sdo.tck.framework.annotations.TCKTest;
import com.ibm.sdo.tck.testcasebase.AnnotationsTestBase;

import commonj.sdo.DataObject;
import commonj.sdo.Property;
import commonj.sdo.Type;

@TCKTest()
public class AnnotationsTest extends AnnotationsTestBase 
{
    @Test
    public void testGlobalPropertySDOXMLName()
    {
        //XSDHelper
        Property p = scope.getXSDHelper().getGlobalProperty(NS,"namefoo1",true);
        assertNotNull(p);
        p = scope.getXSDHelper().getGlobalProperty(NS,"namefoo2",true);
        assertNull(p);
        
        //TypeHelper
        p = scope.getTypeHelper().getOpenContentProperty(NS, "namefoo1");
        assertNull(p);
        p = scope.getTypeHelper().getOpenContentProperty(NS, "namefoo2");
        assertNotNull(p);
    }

    @Test
    public void testSDOXMLName()
    {
        Type t = scope.getTypeHelper().getType(NS,"namefoo3");
        Property p = t.getProperty("namebar1");
        assertNull(p);
        p = t.getProperty("namebar2");
        assertNotNull(p);
    }
    
    @Test
    public void testSDOXMLString()
    {
        Type t = scope.getTypeHelper().getType(NS,"strfoo1");
        Type strType = scope.getTypeHelper().getType("commonj.sdo","String");
        Type intType = scope.getTypeHelper().getType("commonj.sdo","Int");
        Property p = t.getProperty("strbar1");
        assertNotSame(intType,p.getType());
        assertSame(strType,p.getType());
    }
    
    @Test
    public void testSDOXMLDataType()
    {
        Type t = scope.getTypeHelper().getType(NS,"dtfoo1");
        Property p = t.getProperty("dtbar1");
        Type intType = scope.getTypeHelper().getType("commonj.sdo","Int");
        Type longType = scope.getTypeHelper().getType("commonj.sdo","Long");
        assertNotSame(intType,p.getType());
        assertSame(longType,p.getType());
    }
    
    @Test
    public void testSDOJavaInstanceClass()
    {
        Type t = scope.getTypeHelper().getType(NS,"instclassfoo1");
        assertNotNull(t);
        assertEquals(AnnotationsTestInstClass.class, t.getInstanceClass());
        Type t2 = scope.getTypeHelper().getType(NS,"instclassfoo2");
        DataObject dobj = scope.getDataFactory().create(t2);
        assertNotNull(dobj);
        dobj.set("instclassbar1", new AnnotationsTestInstClass("testvalue"));
        assertEquals("testvalue",dobj.get("instclassbar1").toString());
    }
    
    @Before
    public void setUp() throws Exception 
    {
        super.setUp();

        URL url = getClass().getResource(TEST_MODEL);
        InputStream inputStream = url.openStream();
        scope.getXSDHelper().define(inputStream, url.toString());
        inputStream.close();
    }

}
