/**
 *
 *  Licensed to the Apache Software Foundation (ASF) under one
 *  or more contributor license agreements.  See the NOTICE file
 *  distributed with this work for additional information
 *  regarding copyright ownership.  The ASF licenses this file
 *  to you under the Apache License, Version 2.0 (the
 *  "License"); you may not use this file except in compliance
 *  with the License.  You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an
 *  "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 *  KIND, either express or implied.  See the License for the
 *  specific language governing permissions and limitations
 *  under the License.
 */

package com.ibm.sdo.tck.tests.core;

import static org.junit.Assert.assertEquals;

import java.math.BigDecimal;
import java.math.BigInteger;
import java.util.Date;


import org.junit.Test;

import com.ibm.sdo.tck.framework.annotations.Operation;
import com.ibm.sdo.tck.framework.annotations.TCKTest;
import com.ibm.sdo.tck.testcasebase.ConversionTestBase;

import commonj.sdo.DataObject;
import commonj.sdo.Property;
import commonj.sdo.Type;

@TCKTest(testFor = {
        @Operation(className="DataObject", methodName="getBoolean", signature="String", precondition="Property's Type=Boolean"),
        @Operation(className="DataObject", methodName="getString", signature="String", precondition="Property's Type=Boolean"),
        @Operation(className="DataObject", methodName="getByte", signature="String", precondition="Property's Type=Byte"),
        @Operation(className="DataObject", methodName="getDouble", signature="String", precondition="Property's Type=Byte"),
        @Operation(className="DataObject", methodName="getFloat", signature="String", precondition="Property's Type=Byte"),
        @Operation(className="DataObject", methodName="getInt", signature="String", precondition="Property's Type=Byte"),
        @Operation(className="DataObject", methodName="getLong", signature="String", precondition="Property's Type=Byte"),
        @Operation(className="DataObject", methodName="getShort", signature="String", precondition="Property's Type=Byte"),
        @Operation(className="DataObject", methodName="getString", signature="String", precondition="Property's Type=Byte"),
        @Operation(className="DataObject", methodName="getCharacter", signature="String", precondition="Property's Type=Character"),
        @Operation(className="DataObject", methodName="getString", signature="String", precondition="Property's Type=Character"),
        @Operation(className="DataObject", methodName="getByte", signature="String", precondition="Property's Type=Double"),
        @Operation(className="DataObject", methodName="getDouble", signature="String", precondition="Property's Type=Double"),
        @Operation(className="DataObject", methodName="getFloat", signature="String", precondition="Property's Type=Double"),
        @Operation(className="DataObject", methodName="getInt", signature="String", precondition="Property's Type=Double"),
        @Operation(className="DataObject", methodName="getLong", signature="String", precondition="Property's Type=Double"),
        @Operation(className="DataObject", methodName="getShort", signature="String", precondition="Property's Type=Double"),
        @Operation(className="DataObject", methodName="getString", signature="String", precondition="Property's Type=Double"),
        @Operation(className="DataObject", methodName="getDecimal", signature="String", precondition="Property's Type=Double"),
        @Operation(className="DataObject", methodName="getInteger", signature="String", precondition="Property's Type=Double"),
        @Operation(className="DataObject", methodName="getByte", signature="String", precondition="Property's Type=Float"),
        @Operation(className="DataObject", methodName="getDouble", signature="String", precondition="Property's Type=Float"),
        @Operation(className="DataObject", methodName="getFloat", signature="String", precondition="Property's Type=Float"),
        @Operation(className="DataObject", methodName="getInt", signature="String", precondition="Property's Type=Float"),
        @Operation(className="DataObject", methodName="getLong", signature="String", precondition="Property's Type=Float"),
        @Operation(className="DataObject", methodName="getShort", signature="String", precondition="Property's Type=Float"),
        @Operation(className="DataObject", methodName="getString", signature="String", precondition="Property's Type=Float"),
        @Operation(className="DataObject", methodName="getDecimal", signature="String", precondition="Property's Type=Float"),
        @Operation(className="DataObject", methodName="getInteger", signature="String", precondition="Property's Type=Float"),
        @Operation(className="DataObject", methodName="getByte", signature="String", precondition="Property's Type=Int"),
        @Operation(className="DataObject", methodName="getDouble", signature="String", precondition="Property's Type=Int"),
        @Operation(className="DataObject", methodName="getFloat", signature="String", precondition="Property's Type=Int"),
        @Operation(className="DataObject", methodName="getInt", signature="String", precondition="Property's Type=Int"),
        @Operation(className="DataObject", methodName="getLong", signature="String", precondition="Property's Type=Int"),
        @Operation(className="DataObject", methodName="getShort", signature="String", precondition="Property's Type=Int"),
        @Operation(className="DataObject", methodName="getString", signature="String", precondition="Property's Type=Int"),
        @Operation(className="DataObject", methodName="getDecimal", signature="String", precondition="Property's Type=Int"),
        @Operation(className="DataObject", methodName="getInteger", signature="String", precondition="Property's Type=Int"),
        @Operation(className="DataObject", methodName="getByte", signature="String", precondition="Property's Type=Long"),
        @Operation(className="DataObject", methodName="getDouble", signature="String", precondition="Property's Type=Long"),
        @Operation(className="DataObject", methodName="getFloat", signature="String", precondition="Property's Type=Long"),
        @Operation(className="DataObject", methodName="getInt", signature="String", precondition="Property's Type=Long"),
        @Operation(className="DataObject", methodName="getLong", signature="String", precondition="Property's Type=Long"),
        @Operation(className="DataObject", methodName="getShort", signature="String", precondition="Property's Type=Long"),
        @Operation(className="DataObject", methodName="getString", signature="String", precondition="Property's Type=Long"),
        @Operation(className="DataObject", methodName="getDecimal", signature="String", precondition="Property's Type=Long"),
        @Operation(className="DataObject", methodName="getInteger", signature="String", precondition="Property's Type=Long"),
        @Operation(className="DataObject", methodName="getDate", signature="String", precondition="Property's Type=Long"),
        @Operation(className="DataObject", methodName="getByte", signature="String", precondition="Property's Type=Short"),
        @Operation(className="DataObject", methodName="getDouble", signature="String", precondition="Property's Type=Short"),
        @Operation(className="DataObject", methodName="getFloat", signature="String", precondition="Property's Type=Short"),
        @Operation(className="DataObject", methodName="getInt", signature="String", precondition="Property's Type=Short"),
        @Operation(className="DataObject", methodName="getLong", signature="String", precondition="Property's Type=Short"),
        @Operation(className="DataObject", methodName="getShort", signature="String", precondition="Property's Type=Short"),
        @Operation(className="DataObject", methodName="getString", signature="String", precondition="Property's Type=Short"),
        @Operation(className="DataObject", methodName="getBoolean", signature="String", precondition="Property's Type=String"),
        @Operation(className="DataObject", methodName="getByte", signature="String", precondition="Property's Type=String"),
        @Operation(className="DataObject", methodName="getCharacter", signature="String", precondition="Property's Type=String"),
        @Operation(className="DataObject", methodName="getDouble", signature="String", precondition="Property's Type=String"),
        @Operation(className="DataObject", methodName="getFloat", signature="String", precondition="Property's Type=String"),
        @Operation(className="DataObject", methodName="getInt", signature="String", precondition="Property's Type=String"),
        @Operation(className="DataObject", methodName="getLong", signature="String", precondition="Property's Type=String"),
        @Operation(className="DataObject", methodName="getShort", signature="String", precondition="Property's Type=String"),
        @Operation(className="DataObject", methodName="getString", signature="String", precondition="Property's Type=String"),
        @Operation(className="DataObject", methodName="getBytes", signature="String", precondition="Property's Type=String"),
        @Operation(className="DataObject", methodName="getDecimal", signature="String", precondition="Property's Type=String"),
        @Operation(className="DataObject", methodName="getInteger", signature="String", precondition="Property's Type=String"),
        @Operation(className="DataObject", methodName="getDate", signature="String", precondition="Property's Type=String"),
        @Operation(className="DataObject", methodName="getString", signature="String", precondition="Property's Type=Bytes"),
        @Operation(className="DataObject", methodName="getBytes", signature="String", precondition="Property's Type=Bytes"),
        @Operation(className="DataObject", methodName="getInteger", signature="String", precondition="Property's Type=Bytes"),
        @Operation(className="DataObject", methodName="getDouble", signature="String", precondition="Property's Type=Decimal"),
        @Operation(className="DataObject", methodName="getFloat", signature="String", precondition="Property's Type=Decimal"),
        @Operation(className="DataObject", methodName="getInt", signature="String", precondition="Property's Type=Decimal"),
        @Operation(className="DataObject", methodName="getLong", signature="String", precondition="Property's Type=Decimal"),
        @Operation(className="DataObject", methodName="getString", signature="String", precondition="Property's Type=Decimal"),
        @Operation(className="DataObject", methodName="getDecimal", signature="String", precondition="Property's Type=Decimal"),
        @Operation(className="DataObject", methodName="getInteger", signature="String", precondition="Property's Type=Decimal"),
        @Operation(className="DataObject", methodName="getDouble", signature="String", precondition="Property's Type=Integer"),
        @Operation(className="DataObject", methodName="getFloat", signature="String", precondition="Property's Type=Integer"),
        @Operation(className="DataObject", methodName="getInt", signature="String", precondition="Property's Type=Integer"),
        @Operation(className="DataObject", methodName="getLong", signature="String", precondition="Property's Type=Integer"),
        @Operation(className="DataObject", methodName="getString", signature="String", precondition="Property's Type=Integer"),
        @Operation(className="DataObject", methodName="getBytes", signature="String", precondition="Property's Type=Integer"),
        @Operation(className="DataObject", methodName="getDecimal", signature="String", precondition="Property's Type=Integer"),
        @Operation(className="DataObject", methodName="getInteger", signature="String", precondition="Property's Type=Integer"),
        @Operation(className="DataObject", methodName="getLong", signature="String", precondition="Property's Type=Date"),
        @Operation(className="DataObject", methodName="getString", signature="String", precondition="Property's Type=Date"),
        @Operation(className="DataObject", methodName="getDate", signature="String", precondition="Property's Type=Date"),
        @Operation(className="DataObject", methodName="getBoolean", signature="int", precondition="Property's Type=Boolean"),
        @Operation(className="DataObject", methodName="getString", signature="int", precondition="Property's Type=Boolean"),
        @Operation(className="DataObject", methodName="getByte", signature="int", precondition="Property's Type=Byte"),
        @Operation(className="DataObject", methodName="getDouble", signature="int", precondition="Property's Type=Byte"),
        @Operation(className="DataObject", methodName="getFloat", signature="int", precondition="Property's Type=Byte"),
        @Operation(className="DataObject", methodName="getInt", signature="int", precondition="Property's Type=Byte"),
        @Operation(className="DataObject", methodName="getLong", signature="int", precondition="Property's Type=Byte"),
        @Operation(className="DataObject", methodName="getShort", signature="int", precondition="Property's Type=Byte"),
        @Operation(className="DataObject", methodName="getString", signature="int", precondition="Property's Type=Byte"),
        @Operation(className="DataObject", methodName="getCharacter", signature="int", precondition="Property's Type=Character"),
        @Operation(className="DataObject", methodName="getString", signature="int", precondition="Property's Type=Character"),
        @Operation(className="DataObject", methodName="getByte", signature="int", precondition="Property's Type=Double"),
        @Operation(className="DataObject", methodName="getDouble", signature="int", precondition="Property's Type=Double"),
        @Operation(className="DataObject", methodName="getFloat", signature="int", precondition="Property's Type=Double"),
        @Operation(className="DataObject", methodName="getInt", signature="int", precondition="Property's Type=Double"),
        @Operation(className="DataObject", methodName="getLong", signature="int", precondition="Property's Type=Double"),
        @Operation(className="DataObject", methodName="getShort", signature="int", precondition="Property's Type=Double"),
        @Operation(className="DataObject", methodName="getString", signature="int", precondition="Property's Type=Double"),
        @Operation(className="DataObject", methodName="getDecimal", signature="int", precondition="Property's Type=Double"),
        @Operation(className="DataObject", methodName="getInteger", signature="int", precondition="Property's Type=Double"),
        @Operation(className="DataObject", methodName="getByte", signature="int", precondition="Property's Type=Float"),
        @Operation(className="DataObject", methodName="getDouble", signature="int", precondition="Property's Type=Float"),
        @Operation(className="DataObject", methodName="getFloat", signature="int", precondition="Property's Type=Float"),
        @Operation(className="DataObject", methodName="getInt", signature="int", precondition="Property's Type=Float"),
        @Operation(className="DataObject", methodName="getLong", signature="int", precondition="Property's Type=Float"),
        @Operation(className="DataObject", methodName="getShort", signature="int", precondition="Property's Type=Float"),
        @Operation(className="DataObject", methodName="getString", signature="int", precondition="Property's Type=Float"),
        @Operation(className="DataObject", methodName="getDecimal", signature="int", precondition="Property's Type=Float"),
        @Operation(className="DataObject", methodName="getInteger", signature="int", precondition="Property's Type=Float"),
        @Operation(className="DataObject", methodName="getByte", signature="int", precondition="Property's Type=Int"),
        @Operation(className="DataObject", methodName="getDouble", signature="int", precondition="Property's Type=Int"),
        @Operation(className="DataObject", methodName="getFloat", signature="int", precondition="Property's Type=Int"),
        @Operation(className="DataObject", methodName="getInt", signature="int", precondition="Property's Type=Int"),
        @Operation(className="DataObject", methodName="getLong", signature="int", precondition="Property's Type=Int"),
        @Operation(className="DataObject", methodName="getShort", signature="int", precondition="Property's Type=Int"),
        @Operation(className="DataObject", methodName="getString", signature="int", precondition="Property's Type=Int"),
        @Operation(className="DataObject", methodName="getDecimal", signature="int", precondition="Property's Type=Int"),
        @Operation(className="DataObject", methodName="getInteger", signature="int", precondition="Property's Type=Int"),
        @Operation(className="DataObject", methodName="getByte", signature="int", precondition="Property's Type=Long"),
        @Operation(className="DataObject", methodName="getDouble", signature="int", precondition="Property's Type=Long"),
        @Operation(className="DataObject", methodName="getFloat", signature="int", precondition="Property's Type=Long"),
        @Operation(className="DataObject", methodName="getInt", signature="int", precondition="Property's Type=Long"),
        @Operation(className="DataObject", methodName="getLong", signature="int", precondition="Property's Type=Long"),
        @Operation(className="DataObject", methodName="getShort", signature="int", precondition="Property's Type=Long"),
        @Operation(className="DataObject", methodName="getString", signature="int", precondition="Property's Type=Long"),
        @Operation(className="DataObject", methodName="getDecimal", signature="int", precondition="Property's Type=Long"),
        @Operation(className="DataObject", methodName="getInteger", signature="int", precondition="Property's Type=Long"),
        @Operation(className="DataObject", methodName="getDate", signature="int", precondition="Property's Type=Long"),
        @Operation(className="DataObject", methodName="getByte", signature="int", precondition="Property's Type=Short"),
        @Operation(className="DataObject", methodName="getDouble", signature="int", precondition="Property's Type=Short"),
        @Operation(className="DataObject", methodName="getFloat", signature="int", precondition="Property's Type=Short"),
        @Operation(className="DataObject", methodName="getInt", signature="int", precondition="Property's Type=Short"),
        @Operation(className="DataObject", methodName="getLong", signature="int", precondition="Property's Type=Short"),
        @Operation(className="DataObject", methodName="getShort", signature="int", precondition="Property's Type=Short"),
        @Operation(className="DataObject", methodName="getString", signature="int", precondition="Property's Type=Short"),
        @Operation(className="DataObject", methodName="getBoolean", signature="int", precondition="Property's Type=String"),
        @Operation(className="DataObject", methodName="getByte", signature="int", precondition="Property's Type=String"),
        @Operation(className="DataObject", methodName="getCharacter", signature="int", precondition="Property's Type=String"),
        @Operation(className="DataObject", methodName="getDouble", signature="int", precondition="Property's Type=String"),
        @Operation(className="DataObject", methodName="getFloat", signature="int", precondition="Property's Type=String"),
        @Operation(className="DataObject", methodName="getInt", signature="int", precondition="Property's Type=String"),
        @Operation(className="DataObject", methodName="getLong", signature="int", precondition="Property's Type=String"),
        @Operation(className="DataObject", methodName="getShort", signature="int", precondition="Property's Type=String"),
        @Operation(className="DataObject", methodName="getString", signature="int", precondition="Property's Type=String"),
        @Operation(className="DataObject", methodName="getBytes", signature="int", precondition="Property's Type=String"),
        @Operation(className="DataObject", methodName="getDecimal", signature="int", precondition="Property's Type=String"),
        @Operation(className="DataObject", methodName="getInteger", signature="int", precondition="Property's Type=String"),
        @Operation(className="DataObject", methodName="getDate", signature="int", precondition="Property's Type=String"),
        @Operation(className="DataObject", methodName="getString", signature="int", precondition="Property's Type=Bytes"),
        @Operation(className="DataObject", methodName="getBytes", signature="int", precondition="Property's Type=Bytes"),
        @Operation(className="DataObject", methodName="getInteger", signature="int", precondition="Property's Type=Bytes"),
        @Operation(className="DataObject", methodName="getDouble", signature="int", precondition="Property's Type=Decimal"),
        @Operation(className="DataObject", methodName="getFloat", signature="int", precondition="Property's Type=Decimal"),
        @Operation(className="DataObject", methodName="getInt", signature="int", precondition="Property's Type=Decimal"),
        @Operation(className="DataObject", methodName="getLong", signature="int", precondition="Property's Type=Decimal"),
        @Operation(className="DataObject", methodName="getString", signature="int", precondition="Property's Type=Decimal"),
        @Operation(className="DataObject", methodName="getDecimal", signature="int", precondition="Property's Type=Decimal"),
        @Operation(className="DataObject", methodName="getInteger", signature="int", precondition="Property's Type=Decimal"),
        @Operation(className="DataObject", methodName="getDouble", signature="int", precondition="Property's Type=Integer"),
        @Operation(className="DataObject", methodName="getFloat", signature="int", precondition="Property's Type=Integer"),
        @Operation(className="DataObject", methodName="getInt", signature="int", precondition="Property's Type=Integer"),
        @Operation(className="DataObject", methodName="getLong", signature="int", precondition="Property's Type=Integer"),
        @Operation(className="DataObject", methodName="getString", signature="int", precondition="Property's Type=Integer"),
        @Operation(className="DataObject", methodName="getBytes", signature="int", precondition="Property's Type=Integer"),
        @Operation(className="DataObject", methodName="getDecimal", signature="int", precondition="Property's Type=Integer"),
        @Operation(className="DataObject", methodName="getInteger", signature="int", precondition="Property's Type=Integer"),
        @Operation(className="DataObject", methodName="getLong", signature="int", precondition="Property's Type=Date"),
        @Operation(className="DataObject", methodName="getString", signature="int", precondition="Property's Type=Date"),
        @Operation(className="DataObject", methodName="getDate", signature="int", precondition="Property's Type=Date"),
        @Operation(className="DataObject", methodName="getBoolean", signature="property", precondition="Property's Type=Boolean"),
        @Operation(className="DataObject", methodName="getString", signature="property", precondition="Property's Type=Boolean"),
        @Operation(className="DataObject", methodName="getByte", signature="property", precondition="Property's Type=Byte"),
        @Operation(className="DataObject", methodName="getDouble", signature="property", precondition="Property's Type=Byte"),
        @Operation(className="DataObject", methodName="getFloat", signature="property", precondition="Property's Type=Byte"),
        @Operation(className="DataObject", methodName="getInt", signature="property", precondition="Property's Type=Byte"),
        @Operation(className="DataObject", methodName="getLong", signature="property", precondition="Property's Type=Byte"),
        @Operation(className="DataObject", methodName="getShort", signature="property", precondition="Property's Type=Byte"),
        @Operation(className="DataObject", methodName="getString", signature="property", precondition="Property's Type=Byte"),
        @Operation(className="DataObject", methodName="getCharacter", signature="property", precondition="Property's Type=Character"),
        @Operation(className="DataObject", methodName="getString", signature="property", precondition="Property's Type=Character"),
        @Operation(className="DataObject", methodName="getByte", signature="property", precondition="Property's Type=Double"),
        @Operation(className="DataObject", methodName="getDouble", signature="property", precondition="Property's Type=Double"),
        @Operation(className="DataObject", methodName="getFloat", signature="property", precondition="Property's Type=Double"),
        @Operation(className="DataObject", methodName="getInt", signature="property", precondition="Property's Type=Double"),
        @Operation(className="DataObject", methodName="getLong", signature="property", precondition="Property's Type=Double"),
        @Operation(className="DataObject", methodName="getShort", signature="property", precondition="Property's Type=Double"),
        @Operation(className="DataObject", methodName="getString", signature="property", precondition="Property's Type=Double"),
        @Operation(className="DataObject", methodName="getDecimal", signature="property", precondition="Property's Type=Double"),
        @Operation(className="DataObject", methodName="getInteger", signature="property", precondition="Property's Type=Double"),
        @Operation(className="DataObject", methodName="getByte", signature="property", precondition="Property's Type=Float"),
        @Operation(className="DataObject", methodName="getDouble", signature="property", precondition="Property's Type=Float"),
        @Operation(className="DataObject", methodName="getFloat", signature="property", precondition="Property's Type=Float"),
        @Operation(className="DataObject", methodName="getInt", signature="property", precondition="Property's Type=Float"),
        @Operation(className="DataObject", methodName="getLong", signature="property", precondition="Property's Type=Float"),
        @Operation(className="DataObject", methodName="getShort", signature="property", precondition="Property's Type=Float"),
        @Operation(className="DataObject", methodName="getString", signature="property", precondition="Property's Type=Float"),
        @Operation(className="DataObject", methodName="getDecimal", signature="property", precondition="Property's Type=Float"),
        @Operation(className="DataObject", methodName="getInteger", signature="property", precondition="Property's Type=Float"),
        @Operation(className="DataObject", methodName="getByte", signature="property", precondition="Property's Type=Int"),
        @Operation(className="DataObject", methodName="getDouble", signature="property", precondition="Property's Type=Int"),
        @Operation(className="DataObject", methodName="getFloat", signature="property", precondition="Property's Type=Int"),
        @Operation(className="DataObject", methodName="getInt", signature="property", precondition="Property's Type=Int"),
        @Operation(className="DataObject", methodName="getLong", signature="property", precondition="Property's Type=Int"),
        @Operation(className="DataObject", methodName="getShort", signature="property", precondition="Property's Type=Int"),
        @Operation(className="DataObject", methodName="getString", signature="property", precondition="Property's Type=Int"),
        @Operation(className="DataObject", methodName="getDecimal", signature="property", precondition="Property's Type=Int"),
        @Operation(className="DataObject", methodName="getInteger", signature="property", precondition="Property's Type=Int"),
        @Operation(className="DataObject", methodName="getByte", signature="property", precondition="Property's Type=Long"),
        @Operation(className="DataObject", methodName="getDouble", signature="property", precondition="Property's Type=Long"),
        @Operation(className="DataObject", methodName="getFloat", signature="property", precondition="Property's Type=Long"),
        @Operation(className="DataObject", methodName="getInt", signature="property", precondition="Property's Type=Long"),
        @Operation(className="DataObject", methodName="getLong", signature="property", precondition="Property's Type=Long"),
        @Operation(className="DataObject", methodName="getShort", signature="property", precondition="Property's Type=Long"),
        @Operation(className="DataObject", methodName="getString", signature="property", precondition="Property's Type=Long"),
        @Operation(className="DataObject", methodName="getDecimal", signature="property", precondition="Property's Type=Long"),
        @Operation(className="DataObject", methodName="getInteger", signature="property", precondition="Property's Type=Long"),
        @Operation(className="DataObject", methodName="getDate", signature="property", precondition="Property's Type=Long"),
        @Operation(className="DataObject", methodName="getByte", signature="property", precondition="Property's Type=Short"),
        @Operation(className="DataObject", methodName="getDouble", signature="property", precondition="Property's Type=Short"),
        @Operation(className="DataObject", methodName="getFloat", signature="property", precondition="Property's Type=Short"),
        @Operation(className="DataObject", methodName="getInt", signature="property", precondition="Property's Type=Short"),
        @Operation(className="DataObject", methodName="getLong", signature="property", precondition="Property's Type=Short"),
        @Operation(className="DataObject", methodName="getShort", signature="property", precondition="Property's Type=Short"),
        @Operation(className="DataObject", methodName="getString", signature="property", precondition="Property's Type=Short"),
        @Operation(className="DataObject", methodName="getBoolean", signature="property", precondition="Property's Type=String"),
        @Operation(className="DataObject", methodName="getByte", signature="property", precondition="Property's Type=String"),
        @Operation(className="DataObject", methodName="getCharacter", signature="property", precondition="Property's Type=String"),
        @Operation(className="DataObject", methodName="getDouble", signature="property", precondition="Property's Type=String"),
        @Operation(className="DataObject", methodName="getFloat", signature="property", precondition="Property's Type=String"),
        @Operation(className="DataObject", methodName="getInt", signature="property", precondition="Property's Type=String"),
        @Operation(className="DataObject", methodName="getLong", signature="property", precondition="Property's Type=String"),
        @Operation(className="DataObject", methodName="getShort", signature="property", precondition="Property's Type=String"),
        @Operation(className="DataObject", methodName="getString", signature="property", precondition="Property's Type=String"),
        @Operation(className="DataObject", methodName="getBytes", signature="property", precondition="Property's Type=String"),
        @Operation(className="DataObject", methodName="getDecimal", signature="property", precondition="Property's Type=String"),
        @Operation(className="DataObject", methodName="getInteger", signature="property", precondition="Property's Type=String"),
        @Operation(className="DataObject", methodName="getDate", signature="property", precondition="Property's Type=String"),
        @Operation(className="DataObject", methodName="getString", signature="property", precondition="Property's Type=Bytes"),
        @Operation(className="DataObject", methodName="getBytes", signature="property", precondition="Property's Type=Bytes"),
        @Operation(className="DataObject", methodName="getInteger", signature="property", precondition="Property's Type=Bytes"),
        @Operation(className="DataObject", methodName="getDouble", signature="property", precondition="Property's Type=Decimal"),
        @Operation(className="DataObject", methodName="getFloat", signature="property", precondition="Property's Type=Decimal"),
        @Operation(className="DataObject", methodName="getInt", signature="property", precondition="Property's Type=Decimal"),
        @Operation(className="DataObject", methodName="getLong", signature="property", precondition="Property's Type=Decimal"),
        @Operation(className="DataObject", methodName="getString", signature="property", precondition="Property's Type=Decimal"),
        @Operation(className="DataObject", methodName="getDecimal", signature="property", precondition="Property's Type=Decimal"),
        @Operation(className="DataObject", methodName="getInteger", signature="property", precondition="Property's Type=Decimal"),
        @Operation(className="DataObject", methodName="getDouble", signature="property", precondition="Property's Type=Integer"),
        @Operation(className="DataObject", methodName="getFloat", signature="property", precondition="Property's Type=Integer"),
        @Operation(className="DataObject", methodName="getInt", signature="property", precondition="Property's Type=Integer"),
        @Operation(className="DataObject", methodName="getLong", signature="property", precondition="Property's Type=Integer"),
        @Operation(className="DataObject", methodName="getString", signature="property", precondition="Property's Type=Integer"),
        @Operation(className="DataObject", methodName="getBytes", signature="property", precondition="Property's Type=Integer"),
        @Operation(className="DataObject", methodName="getDecimal", signature="property", precondition="Property's Type=Integer"),
        @Operation(className="DataObject", methodName="getInteger", signature="property", precondition="Property's Type=Integer"),
        @Operation(className="DataObject", methodName="getLong", signature="property", precondition="Property's Type=Date"),
        @Operation(className="DataObject", methodName="getString", signature="property", precondition="Property's Type=Date"),
        @Operation(className="DataObject", methodName="getDate", signature="property", precondition="Property's Type=Date"),
        @Operation(className="DataObject", methodName="setBoolean", signature="String,Boolean", precondition="Property's Type=Boolean"),
        @Operation(className="DataObject", methodName="setString", signature="String,String", precondition="Property's Type=Boolean"),
        @Operation(className="DataObject", methodName="setByte", signature="String,Byte", precondition="Property's Type=Byte"),
        @Operation(className="DataObject", methodName="setDouble", signature="String,Double", precondition="Property's Type=Byte"),
        @Operation(className="DataObject", methodName="setFloat", signature="String,Float", precondition="Property's Type=Byte"),
        @Operation(className="DataObject", methodName="setInt", signature="String,Int", precondition="Property's Type=Byte"),
        @Operation(className="DataObject", methodName="setLong", signature="String,Long", precondition="Property's Type=Byte"),
        @Operation(className="DataObject", methodName="setShort", signature="String,Short", precondition="Property's Type=Byte"),
        @Operation(className="DataObject", methodName="setString", signature="String,String", precondition="Property's Type=Byte"),
        @Operation(className="DataObject", methodName="setCharacter", signature="String,Character", precondition="Property's Type=Character"),
        @Operation(className="DataObject", methodName="setString", signature="String,String", precondition="Property's Type=Character"),
        @Operation(className="DataObject", methodName="setByte", signature="String,Byte", precondition="Property's Type=Double"),
        @Operation(className="DataObject", methodName="setDouble", signature="String,Double", precondition="Property's Type=Double"),
        @Operation(className="DataObject", methodName="setFloat", signature="String,Float", precondition="Property's Type=Double"),
        @Operation(className="DataObject", methodName="setInt", signature="String,Int", precondition="Property's Type=Double"),
        @Operation(className="DataObject", methodName="setLong", signature="String,Long", precondition="Property's Type=Double"),
        @Operation(className="DataObject", methodName="setShort", signature="String,Short", precondition="Property's Type=Double"),
        @Operation(className="DataObject", methodName="setString", signature="String,String", precondition="Property's Type=Double"),
        @Operation(className="DataObject", methodName="setDecimal", signature="String,Decimal", precondition="Property's Type=Double"),
        @Operation(className="DataObject", methodName="setInteger", signature="String,Integer", precondition="Property's Type=Double"),
        @Operation(className="DataObject", methodName="setByte", signature="String,Byte", precondition="Property's Type=Float"),
        @Operation(className="DataObject", methodName="setDouble", signature="String,Double", precondition="Property's Type=Float"),
        @Operation(className="DataObject", methodName="setFloat", signature="String,Float", precondition="Property's Type=Float"),
        @Operation(className="DataObject", methodName="setInt", signature="String,Int", precondition="Property's Type=Float"),
        @Operation(className="DataObject", methodName="setLong", signature="String,Long", precondition="Property's Type=Float"),
        @Operation(className="DataObject", methodName="setShort", signature="String,Short", precondition="Property's Type=Float"),
        @Operation(className="DataObject", methodName="setString", signature="String,String", precondition="Property's Type=Float"),
        @Operation(className="DataObject", methodName="setDecimal", signature="String,Decimal", precondition="Property's Type=Float"),
        @Operation(className="DataObject", methodName="setInteger", signature="String,Integer", precondition="Property's Type=Float"),
        @Operation(className="DataObject", methodName="setByte", signature="String,Byte", precondition="Property's Type=Int"),
        @Operation(className="DataObject", methodName="setDouble", signature="String,Double", precondition="Property's Type=Int"),
        @Operation(className="DataObject", methodName="setFloat", signature="String,Float", precondition="Property's Type=Int"),
        @Operation(className="DataObject", methodName="setInt", signature="String,Int", precondition="Property's Type=Int"),
        @Operation(className="DataObject", methodName="setLong", signature="String,Long", precondition="Property's Type=Int"),
        @Operation(className="DataObject", methodName="setShort", signature="String,Short", precondition="Property's Type=Int"),
        @Operation(className="DataObject", methodName="setString", signature="String,String", precondition="Property's Type=Int"),
        @Operation(className="DataObject", methodName="setDecimal", signature="String,Decimal", precondition="Property's Type=Int"),
        @Operation(className="DataObject", methodName="setInteger", signature="String,Integer", precondition="Property's Type=Int"),
        @Operation(className="DataObject", methodName="setByte", signature="String,Byte", precondition="Property's Type=Long"),
        @Operation(className="DataObject", methodName="setDouble", signature="String,Double", precondition="Property's Type=Long"),
        @Operation(className="DataObject", methodName="setFloat", signature="String,Float", precondition="Property's Type=Long"),
        @Operation(className="DataObject", methodName="setInt", signature="String,Int", precondition="Property's Type=Long"),
        @Operation(className="DataObject", methodName="setLong", signature="String,Long", precondition="Property's Type=Long"),
        @Operation(className="DataObject", methodName="setShort", signature="String,Short", precondition="Property's Type=Long"),
        @Operation(className="DataObject", methodName="setString", signature="String,String", precondition="Property's Type=Long"),
        @Operation(className="DataObject", methodName="setDecimal", signature="String,Decimal", precondition="Property's Type=Long"),
        @Operation(className="DataObject", methodName="setInteger", signature="String,Integer", precondition="Property's Type=Long"),
        @Operation(className="DataObject", methodName="setDate", signature="String,Date", precondition="Property's Type=Long"),
        @Operation(className="DataObject", methodName="setByte", signature="String,Byte", precondition="Property's Type=Short"),
        @Operation(className="DataObject", methodName="setDouble", signature="String,Double", precondition="Property's Type=Short"),
        @Operation(className="DataObject", methodName="setFloat", signature="String,Float", precondition="Property's Type=Short"),
        @Operation(className="DataObject", methodName="setInt", signature="String,Int", precondition="Property's Type=Short"),
        @Operation(className="DataObject", methodName="setLong", signature="String,Long", precondition="Property's Type=Short"),
        @Operation(className="DataObject", methodName="setShort", signature="String,Short", precondition="Property's Type=Short"),
        @Operation(className="DataObject", methodName="setString", signature="String,String", precondition="Property's Type=Short"),
        @Operation(className="DataObject", methodName="setBoolean", signature="String,Boolean", precondition="Property's Type=String"),
        @Operation(className="DataObject", methodName="setByte", signature="String,Byte", precondition="Property's Type=String"),
        @Operation(className="DataObject", methodName="setCharacter", signature="String,Character", precondition="Property's Type=String"),
        @Operation(className="DataObject", methodName="setDouble", signature="String,Double", precondition="Property's Type=String"),
        @Operation(className="DataObject", methodName="setFloat", signature="String,Float", precondition="Property's Type=String"),
        @Operation(className="DataObject", methodName="setInt", signature="String,Int", precondition="Property's Type=String"),
        @Operation(className="DataObject", methodName="setLong", signature="String,Long", precondition="Property's Type=String"),
        @Operation(className="DataObject", methodName="setShort", signature="String,Short", precondition="Property's Type=String"),
        @Operation(className="DataObject", methodName="setString", signature="String,String", precondition="Property's Type=String"),
        @Operation(className="DataObject", methodName="setBytes", signature="String,Bytes", precondition="Property's Type=String"),
        @Operation(className="DataObject", methodName="setDecimal", signature="String,Decimal", precondition="Property's Type=String"),
        @Operation(className="DataObject", methodName="setInteger", signature="String,Integer", precondition="Property's Type=String"),
        @Operation(className="DataObject", methodName="setDate", signature="String,Date", precondition="Property's Type=String"),
        @Operation(className="DataObject", methodName="setString", signature="String,String", precondition="Property's Type=Bytes"),
        @Operation(className="DataObject", methodName="setBytes", signature="String,Bytes", precondition="Property's Type=Bytes"),
        @Operation(className="DataObject", methodName="setInteger", signature="String,Integer", precondition="Property's Type=Bytes"),
        @Operation(className="DataObject", methodName="setDouble", signature="String,Double", precondition="Property's Type=Decimal"),
        @Operation(className="DataObject", methodName="setFloat", signature="String,Float", precondition="Property's Type=Decimal"),
        @Operation(className="DataObject", methodName="setInt", signature="String,Int", precondition="Property's Type=Decimal"),
        @Operation(className="DataObject", methodName="setLong", signature="String,Long", precondition="Property's Type=Decimal"),
        @Operation(className="DataObject", methodName="setString", signature="String,String", precondition="Property's Type=Decimal"),
        @Operation(className="DataObject", methodName="setDecimal", signature="String,Decimal", precondition="Property's Type=Decimal"),
        @Operation(className="DataObject", methodName="setInteger", signature="String,Integer", precondition="Property's Type=Decimal"),
        @Operation(className="DataObject", methodName="setDouble", signature="String,Double", precondition="Property's Type=Integer"),
        @Operation(className="DataObject", methodName="setFloat", signature="String,Float", precondition="Property's Type=Integer"),
        @Operation(className="DataObject", methodName="setInt", signature="String,Int", precondition="Property's Type=Integer"),
        @Operation(className="DataObject", methodName="setLong", signature="String,Long", precondition="Property's Type=Integer"),
        @Operation(className="DataObject", methodName="setString", signature="String,String", precondition="Property's Type=Integer"),
        @Operation(className="DataObject", methodName="setBytes", signature="String,Bytes", precondition="Property's Type=Integer"),
        @Operation(className="DataObject", methodName="setDecimal", signature="String,Decimal", precondition="Property's Type=Integer"),
        @Operation(className="DataObject", methodName="setInteger", signature="String,Integer", precondition="Property's Type=Integer"),
        @Operation(className="DataObject", methodName="setLong", signature="String,Long", precondition="Property's Type=Date"),
        @Operation(className="DataObject", methodName="setString", signature="String,String", precondition="Property's Type=Date"),
        @Operation(className="DataObject", methodName="setDate", signature="String,Date", precondition="Property's Type=Date"),
        @Operation(className="DataObject", methodName="setBoolean", signature="int,Boolean", precondition="Property's Type=Boolean"),
        @Operation(className="DataObject", methodName="setString", signature="int,String", precondition="Property's Type=Boolean"),
        @Operation(className="DataObject", methodName="setByte", signature="int,Byte", precondition="Property's Type=Byte"),
        @Operation(className="DataObject", methodName="setDouble", signature="int,Double", precondition="Property's Type=Byte"),
        @Operation(className="DataObject", methodName="setFloat", signature="int,Float", precondition="Property's Type=Byte"),
        @Operation(className="DataObject", methodName="setInt", signature="int,Int", precondition="Property's Type=Byte"),
        @Operation(className="DataObject", methodName="setLong", signature="int,Long", precondition="Property's Type=Byte"),
        @Operation(className="DataObject", methodName="setShort", signature="int,Short", precondition="Property's Type=Byte"),
        @Operation(className="DataObject", methodName="setString", signature="int,String", precondition="Property's Type=Byte"),
        @Operation(className="DataObject", methodName="setCharacter", signature="int,Character", precondition="Property's Type=Character"),
        @Operation(className="DataObject", methodName="setString", signature="int,String", precondition="Property's Type=Character"),
        @Operation(className="DataObject", methodName="setByte", signature="int,Byte", precondition="Property's Type=Double"),
        @Operation(className="DataObject", methodName="setDouble", signature="int,Double", precondition="Property's Type=Double"),
        @Operation(className="DataObject", methodName="setFloat", signature="int,Float", precondition="Property's Type=Double"),
        @Operation(className="DataObject", methodName="setInt", signature="int,Int", precondition="Property's Type=Double"),
        @Operation(className="DataObject", methodName="setLong", signature="int,Long", precondition="Property's Type=Double"),
        @Operation(className="DataObject", methodName="setShort", signature="int,Short", precondition="Property's Type=Double"),
        @Operation(className="DataObject", methodName="setString", signature="int,String", precondition="Property's Type=Double"),
        @Operation(className="DataObject", methodName="setDecimal", signature="int,Decimal", precondition="Property's Type=Double"),
        @Operation(className="DataObject", methodName="setInteger", signature="int,Integer", precondition="Property's Type=Double"),
        @Operation(className="DataObject", methodName="setByte", signature="int,Byte", precondition="Property's Type=Float"),
        @Operation(className="DataObject", methodName="setDouble", signature="int,Double", precondition="Property's Type=Float"),
        @Operation(className="DataObject", methodName="setFloat", signature="int,Float", precondition="Property's Type=Float"),
        @Operation(className="DataObject", methodName="setInt", signature="int,Int", precondition="Property's Type=Float"),
        @Operation(className="DataObject", methodName="setLong", signature="int,Long", precondition="Property's Type=Float"),
        @Operation(className="DataObject", methodName="setShort", signature="int,Short", precondition="Property's Type=Float"),
        @Operation(className="DataObject", methodName="setString", signature="int,String", precondition="Property's Type=Float"),
        @Operation(className="DataObject", methodName="setDecimal", signature="int,Decimal", precondition="Property's Type=Float"),
        @Operation(className="DataObject", methodName="setInteger", signature="int,Integer", precondition="Property's Type=Float"),
        @Operation(className="DataObject", methodName="setByte", signature="int,Byte", precondition="Property's Type=Int"),
        @Operation(className="DataObject", methodName="setDouble", signature="int,Double", precondition="Property's Type=Int"),
        @Operation(className="DataObject", methodName="setFloat", signature="int,Float", precondition="Property's Type=Int"),
        @Operation(className="DataObject", methodName="setInt", signature="int,Int", precondition="Property's Type=Int"),
        @Operation(className="DataObject", methodName="setLong", signature="int,Long", precondition="Property's Type=Int"),
        @Operation(className="DataObject", methodName="setShort", signature="int,Short", precondition="Property's Type=Int"),
        @Operation(className="DataObject", methodName="setString", signature="int,String", precondition="Property's Type=Int"),
        @Operation(className="DataObject", methodName="setDecimal", signature="int,Decimal", precondition="Property's Type=Int"),
        @Operation(className="DataObject", methodName="setInteger", signature="int,Integer", precondition="Property's Type=Int"),
        @Operation(className="DataObject", methodName="setByte", signature="int,Byte", precondition="Property's Type=Long"),
        @Operation(className="DataObject", methodName="setDouble", signature="int,Double", precondition="Property's Type=Long"),
        @Operation(className="DataObject", methodName="setFloat", signature="int,Float", precondition="Property's Type=Long"),
        @Operation(className="DataObject", methodName="setInt", signature="int,Int", precondition="Property's Type=Long"),
        @Operation(className="DataObject", methodName="setLong", signature="int,Long", precondition="Property's Type=Long"),
        @Operation(className="DataObject", methodName="setShort", signature="int,Short", precondition="Property's Type=Long"),
        @Operation(className="DataObject", methodName="setString", signature="int,String", precondition="Property's Type=Long"),
        @Operation(className="DataObject", methodName="setDecimal", signature="int,Decimal", precondition="Property's Type=Long"),
        @Operation(className="DataObject", methodName="setInteger", signature="int,Integer", precondition="Property's Type=Long"),
        @Operation(className="DataObject", methodName="setDate", signature="int,Date", precondition="Property's Type=Long"),
        @Operation(className="DataObject", methodName="setByte", signature="int,Byte", precondition="Property's Type=Short"),
        @Operation(className="DataObject", methodName="setDouble", signature="int,Double", precondition="Property's Type=Short"),
        @Operation(className="DataObject", methodName="setFloat", signature="int,Float", precondition="Property's Type=Short"),
        @Operation(className="DataObject", methodName="setInt", signature="int,Int", precondition="Property's Type=Short"),
        @Operation(className="DataObject", methodName="setLong", signature="int,Long", precondition="Property's Type=Short"),
        @Operation(className="DataObject", methodName="setShort", signature="int,Short", precondition="Property's Type=Short"),
        @Operation(className="DataObject", methodName="setString", signature="int,String", precondition="Property's Type=Short"),
        @Operation(className="DataObject", methodName="setBoolean", signature="int,Boolean", precondition="Property's Type=String"),
        @Operation(className="DataObject", methodName="setByte", signature="int,Byte", precondition="Property's Type=String"),
        @Operation(className="DataObject", methodName="setCharacter", signature="int,Character", precondition="Property's Type=String"),
        @Operation(className="DataObject", methodName="setDouble", signature="int,Double", precondition="Property's Type=String"),
        @Operation(className="DataObject", methodName="setFloat", signature="int,Float", precondition="Property's Type=String"),
        @Operation(className="DataObject", methodName="setInt", signature="int,Int", precondition="Property's Type=String"),
        @Operation(className="DataObject", methodName="setLong", signature="int,Long", precondition="Property's Type=String"),
        @Operation(className="DataObject", methodName="setShort", signature="int,Short", precondition="Property's Type=String"),
        @Operation(className="DataObject", methodName="setString", signature="int,String", precondition="Property's Type=String"),
        @Operation(className="DataObject", methodName="setBytes", signature="int,Bytes", precondition="Property's Type=String"),
        @Operation(className="DataObject", methodName="setDecimal", signature="int,Decimal", precondition="Property's Type=String"),
        @Operation(className="DataObject", methodName="setInteger", signature="int,Integer", precondition="Property's Type=String"),
        @Operation(className="DataObject", methodName="setDate", signature="int,Date", precondition="Property's Type=String"),
        @Operation(className="DataObject", methodName="setString", signature="int,String", precondition="Property's Type=Bytes"),
        @Operation(className="DataObject", methodName="setBytes", signature="int,Bytes", precondition="Property's Type=Bytes"),
        @Operation(className="DataObject", methodName="setInteger", signature="int,Integer", precondition="Property's Type=Bytes"),
        @Operation(className="DataObject", methodName="setDouble", signature="int,Double", precondition="Property's Type=Decimal"),
        @Operation(className="DataObject", methodName="setFloat", signature="int,Float", precondition="Property's Type=Decimal"),
        @Operation(className="DataObject", methodName="setInt", signature="int,Int", precondition="Property's Type=Decimal"),
        @Operation(className="DataObject", methodName="setLong", signature="int,Long", precondition="Property's Type=Decimal"),
        @Operation(className="DataObject", methodName="setString", signature="int,String", precondition="Property's Type=Decimal"),
        @Operation(className="DataObject", methodName="setDecimal", signature="int,Decimal", precondition="Property's Type=Decimal"),
        @Operation(className="DataObject", methodName="setInteger", signature="int,Integer", precondition="Property's Type=Decimal"),
        @Operation(className="DataObject", methodName="setDouble", signature="int,Double", precondition="Property's Type=Integer"),
        @Operation(className="DataObject", methodName="setFloat", signature="int,Float", precondition="Property's Type=Integer"),
        @Operation(className="DataObject", methodName="setInt", signature="int,Int", precondition="Property's Type=Integer"),
        @Operation(className="DataObject", methodName="setLong", signature="int,Long", precondition="Property's Type=Integer"),
        @Operation(className="DataObject", methodName="setString", signature="int,String", precondition="Property's Type=Integer"),
        @Operation(className="DataObject", methodName="setBytes", signature="int,Bytes", precondition="Property's Type=Integer"),
        @Operation(className="DataObject", methodName="setDecimal", signature="int,Decimal", precondition="Property's Type=Integer"),
        @Operation(className="DataObject", methodName="setInteger", signature="int,Integer", precondition="Property's Type=Integer"),
        @Operation(className="DataObject", methodName="setLong", signature="int,Long", precondition="Property's Type=Date"),
        @Operation(className="DataObject", methodName="setString", signature="int,String", precondition="Property's Type=Date"),
        @Operation(className="DataObject", methodName="setDate", signature="int,Date", precondition="Property's Type=Date"),
        @Operation(className="DataObject", methodName="setBoolean", signature="String,Boolean", precondition="Property's Type=Boolean"),
        @Operation(className="DataObject", methodName="setString", signature="String,String", precondition="Property's Type=Boolean"),
        @Operation(className="DataObject", methodName="setByte", signature="String,Byte", precondition="Property's Type=Byte"),
        @Operation(className="DataObject", methodName="setDouble", signature="String,Double", precondition="Property's Type=Byte"),
        @Operation(className="DataObject", methodName="setFloat", signature="String,Float", precondition="Property's Type=Byte"),
        @Operation(className="DataObject", methodName="setInt", signature="String,Int", precondition="Property's Type=Byte"),
        @Operation(className="DataObject", methodName="setLong", signature="String,Long", precondition="Property's Type=Byte"),
        @Operation(className="DataObject", methodName="setShort", signature="String,Short", precondition="Property's Type=Byte"),
        @Operation(className="DataObject", methodName="setString", signature="String,String", precondition="Property's Type=Byte"),
        @Operation(className="DataObject", methodName="setCharacter", signature="String,Character", precondition="Property's Type=Character"),
        @Operation(className="DataObject", methodName="setString", signature="String,String", precondition="Property's Type=Character"),
        @Operation(className="DataObject", methodName="setByte", signature="String,Byte", precondition="Property's Type=Double"),
        @Operation(className="DataObject", methodName="setDouble", signature="String,Double", precondition="Property's Type=Double"),
        @Operation(className="DataObject", methodName="setFloat", signature="String,Float", precondition="Property's Type=Double"),
        @Operation(className="DataObject", methodName="setInt", signature="String,Int", precondition="Property's Type=Double"),
        @Operation(className="DataObject", methodName="setLong", signature="String,Long", precondition="Property's Type=Double"),
        @Operation(className="DataObject", methodName="setShort", signature="String,Short", precondition="Property's Type=Double"),
        @Operation(className="DataObject", methodName="setString", signature="String,String", precondition="Property's Type=Double"),
        @Operation(className="DataObject", methodName="setDecimal", signature="String,Decimal", precondition="Property's Type=Double"),
        @Operation(className="DataObject", methodName="setInteger", signature="String,Integer", precondition="Property's Type=Double"),
        @Operation(className="DataObject", methodName="setByte", signature="String,Byte", precondition="Property's Type=Float"),
        @Operation(className="DataObject", methodName="setDouble", signature="String,Double", precondition="Property's Type=Float"),
        @Operation(className="DataObject", methodName="setFloat", signature="String,Float", precondition="Property's Type=Float"),
        @Operation(className="DataObject", methodName="setInt", signature="String,Int", precondition="Property's Type=Float"),
        @Operation(className="DataObject", methodName="setLong", signature="String,Long", precondition="Property's Type=Float"),
        @Operation(className="DataObject", methodName="setShort", signature="String,Short", precondition="Property's Type=Float"),
        @Operation(className="DataObject", methodName="setString", signature="String,String", precondition="Property's Type=Float"),
        @Operation(className="DataObject", methodName="setDecimal", signature="String,Decimal", precondition="Property's Type=Float"),
        @Operation(className="DataObject", methodName="setInteger", signature="String,Integer", precondition="Property's Type=Float"),
        @Operation(className="DataObject", methodName="setByte", signature="String,Byte", precondition="Property's Type=Int"),
        @Operation(className="DataObject", methodName="setDouble", signature="String,Double", precondition="Property's Type=Int"),
        @Operation(className="DataObject", methodName="setFloat", signature="String,Float", precondition="Property's Type=Int"),
        @Operation(className="DataObject", methodName="setInt", signature="String,Int", precondition="Property's Type=Int"),
        @Operation(className="DataObject", methodName="setLong", signature="String,Long", precondition="Property's Type=Int"),
        @Operation(className="DataObject", methodName="setShort", signature="String,Short", precondition="Property's Type=Int"),
        @Operation(className="DataObject", methodName="setString", signature="String,String", precondition="Property's Type=Int"),
        @Operation(className="DataObject", methodName="setDecimal", signature="String,Decimal", precondition="Property's Type=Int"),
        @Operation(className="DataObject", methodName="setInteger", signature="String,Integer", precondition="Property's Type=Int"),
        @Operation(className="DataObject", methodName="setByte", signature="String,Byte", precondition="Property's Type=Long"),
        @Operation(className="DataObject", methodName="setDouble", signature="String,Double", precondition="Property's Type=Long"),
        @Operation(className="DataObject", methodName="setFloat", signature="String,Float", precondition="Property's Type=Long"),
        @Operation(className="DataObject", methodName="setInt", signature="String,Int", precondition="Property's Type=Long"),
        @Operation(className="DataObject", methodName="setLong", signature="String,Long", precondition="Property's Type=Long"),
        @Operation(className="DataObject", methodName="setShort", signature="String,Short", precondition="Property's Type=Long"),
        @Operation(className="DataObject", methodName="setString", signature="String,String", precondition="Property's Type=Long"),
        @Operation(className="DataObject", methodName="setDecimal", signature="String,Decimal", precondition="Property's Type=Long"),
        @Operation(className="DataObject", methodName="setInteger", signature="String,Integer", precondition="Property's Type=Long"),
        @Operation(className="DataObject", methodName="setDate", signature="String,Date", precondition="Property's Type=Long"),
        @Operation(className="DataObject", methodName="setByte", signature="String,Byte", precondition="Property's Type=Short"),
        @Operation(className="DataObject", methodName="setDouble", signature="String,Double", precondition="Property's Type=Short"),
        @Operation(className="DataObject", methodName="setFloat", signature="String,Float", precondition="Property's Type=Short"),
        @Operation(className="DataObject", methodName="setInt", signature="String,Int", precondition="Property's Type=Short"),
        @Operation(className="DataObject", methodName="setLong", signature="String,Long", precondition="Property's Type=Short"),
        @Operation(className="DataObject", methodName="setShort", signature="String,Short", precondition="Property's Type=Short"),
        @Operation(className="DataObject", methodName="setString", signature="String,String", precondition="Property's Type=Short"),
        @Operation(className="DataObject", methodName="setBoolean", signature="String,Boolean", precondition="Property's Type=String"),
        @Operation(className="DataObject", methodName="setByte", signature="String,Byte", precondition="Property's Type=String"),
        @Operation(className="DataObject", methodName="setCharacter", signature="String,Character", precondition="Property's Type=String"),
        @Operation(className="DataObject", methodName="setDouble", signature="String,Double", precondition="Property's Type=String"),
        @Operation(className="DataObject", methodName="setFloat", signature="String,Float", precondition="Property's Type=String"),
        @Operation(className="DataObject", methodName="setInt", signature="String,Int", precondition="Property's Type=String"),
        @Operation(className="DataObject", methodName="setLong", signature="String,Long", precondition="Property's Type=String"),
        @Operation(className="DataObject", methodName="setShort", signature="String,Short", precondition="Property's Type=String"),
        @Operation(className="DataObject", methodName="setString", signature="String,String", precondition="Property's Type=String"),
        @Operation(className="DataObject", methodName="setBytes", signature="String,Bytes", precondition="Property's Type=String"),
        @Operation(className="DataObject", methodName="setDecimal", signature="String,Decimal", precondition="Property's Type=String"),
        @Operation(className="DataObject", methodName="setInteger", signature="String,Integer", precondition="Property's Type=String"),
        @Operation(className="DataObject", methodName="setDate", signature="String,Date", precondition="Property's Type=String"),
        @Operation(className="DataObject", methodName="setString", signature="String,String", precondition="Property's Type=Bytes"),
        @Operation(className="DataObject", methodName="setBytes", signature="String,Bytes", precondition="Property's Type=Bytes"),
        @Operation(className="DataObject", methodName="setInteger", signature="String,Integer", precondition="Property's Type=Bytes"),
        @Operation(className="DataObject", methodName="setDouble", signature="String,Double", precondition="Property's Type=Decimal"),
        @Operation(className="DataObject", methodName="setFloat", signature="String,Float", precondition="Property's Type=Decimal"),
        @Operation(className="DataObject", methodName="setInt", signature="String,Int", precondition="Property's Type=Decimal"),
        @Operation(className="DataObject", methodName="setLong", signature="String,Long", precondition="Property's Type=Decimal"),
        @Operation(className="DataObject", methodName="setString", signature="String,String", precondition="Property's Type=Decimal"),
        @Operation(className="DataObject", methodName="setDecimal", signature="String,Decimal", precondition="Property's Type=Decimal"),
        @Operation(className="DataObject", methodName="setInteger", signature="String,Integer", precondition="Property's Type=Decimal"),
        @Operation(className="DataObject", methodName="setDouble", signature="String,Double", precondition="Property's Type=Integer"),
        @Operation(className="DataObject", methodName="setFloat", signature="String,Float", precondition="Property's Type=Integer"),
        @Operation(className="DataObject", methodName="setInt", signature="String,Int", precondition="Property's Type=Integer"),
        @Operation(className="DataObject", methodName="setLong", signature="String,Long", precondition="Property's Type=Integer"),
        @Operation(className="DataObject", methodName="setString", signature="String,String", precondition="Property's Type=Integer"),
        @Operation(className="DataObject", methodName="setBytes", signature="String,Bytes", precondition="Property's Type=Integer"),
        @Operation(className="DataObject", methodName="setDecimal", signature="String,Decimal", precondition="Property's Type=Integer"),
        @Operation(className="DataObject", methodName="setInteger", signature="String,Integer", precondition="Property's Type=Integer"),
        @Operation(className="DataObject", methodName="setLong", signature="String,Long", precondition="Property's Type=Date"),
        @Operation(className="DataObject", methodName="setString", signature="String,String", precondition="Property's Type=Date"),
        @Operation(className="DataObject", methodName="setDate", signature="String,Date", precondition="Property's Type=Date")        
})
public class DataObjectAccessorTest extends ConversionTestBase
{
    /**
     * Verifies the DataObject.getXXXXX(String) methods. <p/>
     * @throws Exception 
     * @see commonj.sdo.DataObject#getXXXXX(String)
     * @see <a href="http://osoa.org/download/attachments/36/Java-SDO-Spec-v2.1.0-FINAL.pdf?version=1#page=21">2.1 spec section 3.1.12</a>
     * @see <a href="http://osoa.org/download/attachments/36/Java-SDO-Spec-v2.1.0-FINAL.pdf?version=1#page=138">2.1 spec section 16</a>
     */
    @Test
    public void testPropertyGetStringMethods() throws Exception 
    {
        for( int i=0; i<TYPE_CONV_TABLE.length; i++ )
        {
            if( TYPE_CONV_TABLE[i] == 'x' || TYPE_CONV_TABLE[i] == 'X' )
            {
                int fromType   = i/TYPE_NAMES.length;
                int toType = i%TYPE_NAMES.length;
                
                Object originalValue = getTestValue(fromType,toType);
                Type doType = getScope().getTypeHelper().getType(NS,TypeName);
                String propString = TYPE_NAMES[fromType];
                DataObject dobj   = getScope().getDataFactory().create(doType);
                setProperty(dobj, propString, fromType, originalValue);
                Object newVal = getProperty(dobj,propString,toType);
                assertEquals("Conversion did not yield expected value for property type '" + 
                             TYPE_NAMES[fromType] + "' using get" + TYPE_NAMES[toType] + "(String)",
                             0,COMPARE_ANY.compare(originalValue,newVal));
                //System.out.println("@Operation(className=\"DataObject\", methodName=\"get" + 
                //                   TYPE_NAMES[toType] + "\", signature=\"String\", precondition=\"Property's Type=" +
                //                   TYPE_NAMES[fromType] + "\"),");
            }
        }
    }

    /**
     * Verifies the DataObject.getXXXXX(int) methods. <p/>
     * @throws Exception 
     * @see commonj.sdo.DataObject#getXXXXX(int)
     * @see <a href="http://osoa.org/download/attachments/36/Java-SDO-Spec-v2.1.0-FINAL.pdf?version=1#page=21">2.1 spec section 3.1.12</a>
     * @see <a href="http://osoa.org/download/attachments/36/Java-SDO-Spec-v2.1.0-FINAL.pdf?version=1#page=138">2.1 spec section 16</a>
     */
    @Test
    public void testPropertyGetIntMethods() throws Exception 
    {
        for( int i=0; i<TYPE_CONV_TABLE.length; i++ )
        {
            if( TYPE_CONV_TABLE[i] == 'x' || TYPE_CONV_TABLE[i] == 'X' )
            {
                int fromType   = i/TYPE_NAMES.length;
                int toType = i%TYPE_NAMES.length;
                
                Object originalValue = getTestValue(fromType,toType);
                Type doType = getScope().getTypeHelper().getType(NS,TypeName);
                Property prop     = doType.getProperty(TYPE_NAMES[fromType]);
                Integer propIndex = doType.getProperties().indexOf(prop);
                DataObject dobj   = getScope().getDataFactory().create(doType);
                setProperty(dobj, propIndex.intValue(), fromType, originalValue);
                Object newVal = getProperty(dobj,propIndex.intValue(),toType);
                assertEquals("Conversion did not yield expected value for property type '" + 
                             TYPE_NAMES[fromType] + "' using get" + TYPE_NAMES[toType] +"(int)",
                             0,COMPARE_ANY.compare(originalValue,newVal));
                //System.out.println("@Operation(className=\"DataObject\", methodName=\"get" + 
                //                   TYPE_NAMES[toType] + "\", signature=\"int\", precondition=\"Property's Type=" +
                //                   TYPE_NAMES[fromType] + "\"),");
            }
        }
    }
    
    /**
     * Verifies the DataObject.getXXXXX(property) methods. <p/>
     * @throws Exception 
     * @see commonj.sdo.DataObject#getXXXXX(property)
     * @see <a href="http://osoa.org/download/attachments/36/Java-SDO-Spec-v2.1.0-FINAL.pdf?version=1#page=21">2.1 spec section 3.1.12</a>
     * @see <a href="http://osoa.org/download/attachments/36/Java-SDO-Spec-v2.1.0-FINAL.pdf?version=1#page=138">2.1 spec section 16</a>
     */
    @Test
    public void testPropertyGetPropertyMethods() throws Exception 
    {
        for( int i=0; i<TYPE_CONV_TABLE.length; i++ )
        {
            if( TYPE_CONV_TABLE[i] == 'x' || TYPE_CONV_TABLE[i] == 'X' )
            {
                int fromType   = i/TYPE_NAMES.length;
                int toType = i%TYPE_NAMES.length;
                
                Object originalValue = getTestValue(fromType,toType);
                Type doType = getScope().getTypeHelper().getType(NS,TypeName);
                Property prop     = doType.getProperty(TYPE_NAMES[fromType]);
                DataObject dobj   = getScope().getDataFactory().create(doType);
                setProperty(dobj, prop, fromType, originalValue);
                Object newVal = getProperty(dobj,prop,toType);
                assertEquals("Conversion did not yield expected value for property type '" + 
                             TYPE_NAMES[fromType] + "' using get" + TYPE_NAMES[toType] +"(property)",
                             0,COMPARE_ANY.compare(originalValue,newVal));
                //System.out.println("@Operation(className=\"DataObject\", methodName=\"get" + 
                //                   TYPE_NAMES[toType] + "\", signature=\"property\", precondition=\"Property's Type=" +
                //                   TYPE_NAMES[fromType] + "\"),");
            }
        }
    }
    
    /**
     * Verifies the DataObject.setXXXXX(String, XXXXX) methods. <p/>
     * @throws Exception 
     * @see commonj.sdo.DataObject#setXXXXX(String, XXXXX)
     * @see <a href="http://osoa.org/download/attachments/36/Java-SDO-Spec-v2.1.0-FINAL.pdf?version=1#page=21">2.1 spec section 3.1.12</a>
     * @see <a href="http://osoa.org/download/attachments/36/Java-SDO-Spec-v2.1.0-FINAL.pdf?version=1#page=138">2.1 spec section 16</a>
     */
    @Test
    public void testPropertySetStringMethods() throws Exception 
    {
        for( int i=0; i<TYPE_CONV_TABLE.length; i++ )
        {
            if( TYPE_CONV_TABLE[i] == 'x' || TYPE_CONV_TABLE[i] == 'X' )
            {
                int propType   = i/TYPE_NAMES.length;
                int newValType = i%TYPE_NAMES.length;
                
                Object originalValue = getTestValue(newValType,propType);
                Type doType = getScope().getTypeHelper().getType(NS,TypeName);
                String propString = TYPE_NAMES[propType];
                DataObject dobj   = getScope().getDataFactory().create(doType);
                setProperty(dobj, propString, newValType, originalValue);
                Object newVal = getProperty(dobj,propString,propType);
                assertEquals("Conversion did not yield expected value for property type '" + 
                             TYPE_NAMES[propType] + "' using set" + TYPE_NAMES[newValType] + "(String," +
                             TYPE_NAMES[newValType] + ")",
                             0,COMPARE_ANY.compare(originalValue,newVal));
                //System.out.println("@Operation(className=\"DataObject\", methodName=\"set" + 
                //                    TYPE_NAMES[newValType] + "\", signature=\"String,"+TYPE_NAMES[newValType] +
                //                    "\", precondition=\"Property's Type=" + TYPE_NAMES[propType] + "\"),");
            }
        }
    }

    /**
     * Verifies the DataObject.setXXXXX(int, XXXXX) methods. <p/>
     * @throws Exception 
     * @see commonj.sdo.DataObject#setXXXXX(int, XXXXX)
     * @see <a href="http://osoa.org/download/attachments/36/Java-SDO-Spec-v2.1.0-FINAL.pdf?version=1#page=21">2.1 spec section 3.1.12</a>
     * @see <a href="http://osoa.org/download/attachments/36/Java-SDO-Spec-v2.1.0-FINAL.pdf?version=1#page=138">2.1 spec section 16</a>
     */
    @Test
    public void testPropertySetIntMethods() throws Exception 
    {
        for( int i=0; i<TYPE_CONV_TABLE.length; i++ )
        {
            if( TYPE_CONV_TABLE[i] == 'x' || TYPE_CONV_TABLE[i] == 'X' )
            {
                int propType   = i/TYPE_NAMES.length;
                int newValType = i%TYPE_NAMES.length;
                
                Object originalValue = getTestValue(newValType,propType);
                Type doType = getScope().getTypeHelper().getType(NS,TypeName);
                Property prop     = doType.getProperty(TYPE_NAMES[propType]);
                Integer propIndex = doType.getProperties().indexOf(prop);
                DataObject dobj   = getScope().getDataFactory().create(doType);
                setProperty(dobj, propIndex.intValue(), newValType, originalValue);
                Object newVal = getProperty(dobj,propIndex.intValue(),propType);
                assertEquals("Conversion did not yield expected value for property type '" + 
                        TYPE_NAMES[propType] + "' using set" + TYPE_NAMES[newValType] + "(int," +
                        TYPE_NAMES[newValType] + ")",
                        0,COMPARE_ANY.compare(originalValue,newVal));
                //System.out.println("@Operation(className=\"DataObject\", methodName=\"set" + 
                //                   TYPE_NAMES[newValType] + "\", signature=\"int,"+TYPE_NAMES[newValType] +
                //                   "\", precondition=\"Property's Type=" + TYPE_NAMES[propType] + "\"),");
            }
        }
    }
    
    /**
     * Verifies the DataObject.setXXXXX(property, XXXXX) methods. <p/>
     * @throws Exception 
     * @see commonj.sdo.DataObject#setXXXXX(property, XXXXX)
     * @see <a href="http://osoa.org/download/attachments/36/Java-SDO-Spec-v2.1.0-FINAL.pdf?version=1#page=21">2.1 spec section 3.1.12</a>
     * @see <a href="http://osoa.org/download/attachments/36/Java-SDO-Spec-v2.1.0-FINAL.pdf?version=1#page=138">2.1 spec section 16</a>
     */
    @Test
    public void testPropertySetPropertyMethods() throws Exception 
    {
        for( int i=0; i<TYPE_CONV_TABLE.length; i++ )
        {
            if( TYPE_CONV_TABLE[i] == 'x' || TYPE_CONV_TABLE[i] == 'X' )
            {
                int propType   = i/TYPE_NAMES.length;
                int newValType = i%TYPE_NAMES.length;
                
                Object originalValue = getTestValue(newValType,propType);
                Type doType = getScope().getTypeHelper().getType(NS,TypeName);
                Property prop     = doType.getProperty(TYPE_NAMES[propType]);
                DataObject dobj   = getScope().getDataFactory().create(doType);
                setProperty(dobj, prop, newValType, originalValue);
                Object newVal = getProperty(dobj,prop,propType);
                assertEquals("Conversion did not yield expected value for property type '" + 
                        TYPE_NAMES[propType] + "' using set" + TYPE_NAMES[newValType] + "(property," +
                        TYPE_NAMES[newValType] + ")",
                        0,COMPARE_ANY.compare(originalValue,newVal));
                //System.out.println("@Operation(className=\"DataObject\", methodName=\"set" + 
                //                   TYPE_NAMES[newValType] + "\", signature=\"String,"+TYPE_NAMES[newValType] +
                //                   "\", precondition=\"Property's Type=" + TYPE_NAMES[propType] + "\"),");
            }
        }
    }
        
    private Object getProperty(DataObject dobj, Object property, int type )
    {
        Object retVal = null;
        
        
        // ***************************************
        // *** Use String based property getter 
        // ***************************************
        if( property instanceof String )
        {
            switch(type)
            {
                case TYPE_INDEX_BOOLEAN: 
                    retVal = dobj.getBoolean((String)property);
                    break;
                case TYPE_INDEX_BYTE:
                    retVal = dobj.getByte((String)property);
                    break;
                case TYPE_INDEX_CHARACTER:
                    retVal = dobj.getChar((String)property);
                    break;
                case TYPE_INDEX_DOUBLE: 
                    retVal = dobj.getDouble((String)property);
                    break;
                case TYPE_INDEX_FLOAT:
                    retVal = dobj.getFloat(((String)property));
                    break;
                case TYPE_INDEX_INT:  
                    retVal = dobj.getInt((String)property);
                    break;
                case TYPE_INDEX_LONG: 
                    retVal = dobj.getLong((String)property);
                    break;
                case TYPE_INDEX_SHORT: 
                    retVal = dobj.getShort((String)property);
                    break;
                case TYPE_INDEX_STRING:
                    retVal = dobj.getString((String)property);
                    break;
                case TYPE_INDEX_BYTES:
                    retVal = dobj.getBytes((String)property);
                    break;
                case TYPE_INDEX_DECIMAL:
                    retVal = dobj.getBigDecimal((String)property);
                    break;
                case TYPE_INDEX_INTEGER:   
                    retVal = dobj.getBigInteger((String)property);
                    break;
                case TYPE_INDEX_DATE:  
                    retVal = dobj.getDate(((String)property));
                    break;
            }
        }
        // ***************************************
        // *** Use int based property getter 
        // ***************************************
        if( property instanceof Integer )
        {
            switch(type)
            {
                case TYPE_INDEX_BOOLEAN: 
                    retVal = dobj.getBoolean(((Integer)property).intValue());
                    break;
                case TYPE_INDEX_BYTE:
                    retVal = dobj.getByte(((Integer)property).intValue());
                    break;
                case TYPE_INDEX_CHARACTER:
                    retVal = dobj.getChar(((Integer)property).intValue());
                    break;
                case TYPE_INDEX_DOUBLE: 
                    retVal = dobj.getDouble(((Integer)property).intValue());
                    break;
                case TYPE_INDEX_FLOAT:
                    retVal = dobj.getFloat((((Integer)property).intValue()));
                    break;
                case TYPE_INDEX_INT:  
                    retVal = dobj.getInt(((Integer)property).intValue());
                    break;
                case TYPE_INDEX_LONG: 
                    retVal = dobj.getLong(((Integer)property).intValue());
                    break;
                case TYPE_INDEX_SHORT: 
                    retVal = dobj.getShort(((Integer)property).intValue());
                    break;
                case TYPE_INDEX_STRING:
                    retVal = dobj.getString(((Integer)property).intValue());
                    break;
                case TYPE_INDEX_BYTES:
                    retVal = dobj.getBytes(((Integer)property).intValue());
                    break;
                case TYPE_INDEX_DECIMAL:
                    retVal = dobj.getBigDecimal(((Integer)property).intValue());
                    break;
                case TYPE_INDEX_INTEGER:   
                    retVal = dobj.getBigInteger(((Integer)property).intValue());
                    break;
                case TYPE_INDEX_DATE:  
                    retVal = dobj.getDate((((Integer)property).intValue()));
                    break;
            }
        }
        // ***************************************
        // *** Use Property based property getter 
        // ***************************************
        if( property instanceof Property )
        {
            switch(type)
            {
                case TYPE_INDEX_BOOLEAN: 
                    retVal = dobj.getBoolean((Property)property);
                    break;
                case TYPE_INDEX_BYTE:
                    retVal = dobj.getByte((Property)property);
                    break;
                case TYPE_INDEX_CHARACTER:
                    retVal = dobj.getChar((Property)property);
                    break;
                case TYPE_INDEX_DOUBLE: 
                    retVal = dobj.getDouble((Property)property);
                    break;
                case TYPE_INDEX_FLOAT:
                    retVal = dobj.getFloat(((Property)property));
                    break;
                case TYPE_INDEX_INT:  
                    retVal = dobj.getInt((Property)property);
                    break;
                case TYPE_INDEX_LONG: 
                    retVal = dobj.getLong((Property)property);
                    break;
                case TYPE_INDEX_SHORT: 
                    retVal = dobj.getShort((Property)property);
                    break;
                case TYPE_INDEX_STRING:
                    retVal = dobj.getString((Property)property);
                    break;
                case TYPE_INDEX_BYTES:
                    retVal = dobj.getBytes((Property)property);
                    break;
                case TYPE_INDEX_DECIMAL:
                    retVal = dobj.getBigDecimal((Property)property);
                    break;
                case TYPE_INDEX_INTEGER:   
                    retVal = dobj.getBigInteger((Property)property);
                    break;
                case TYPE_INDEX_DATE:  
                    retVal = dobj.getDate(((Property)property));
                    break;
            } 
        }
        
        return retVal;
    }
    
    private void setProperty(DataObject dobj, Object property, int type, Object value )
    {
        // ***************************************
        // *** Use String based property setter 
        // ***************************************
        if( property instanceof String )
        {
            switch(type)
            {
                case TYPE_INDEX_BOOLEAN: 
                    dobj.setBoolean((String)property,((Boolean)value).booleanValue());
                    break;
                case TYPE_INDEX_BYTE:
                    dobj.setByte((String)property, ((Byte)value).byteValue());
                    break;
                case TYPE_INDEX_CHARACTER:
                    dobj.setChar((String)property, ((Character)value).charValue());
                    break;
                case TYPE_INDEX_DOUBLE: 
                    dobj.setDouble((String)property, ((Double)value).doubleValue());
                    break;
                case TYPE_INDEX_FLOAT:
                    dobj.setFloat(((String)property), ((Float)value).floatValue());
                    break;
                case TYPE_INDEX_INT:  
                    dobj.setInt((String)property, ((Integer)value).intValue());
                    break;
                case TYPE_INDEX_LONG: 
                    dobj.setLong((String)property, ((Long)value).longValue());
                    break;
                case TYPE_INDEX_SHORT: 
                    dobj.setShort((String)property, ((Short)value).shortValue());
                    break;
                case TYPE_INDEX_STRING:
                    dobj.setString((String)property, (String)value);
                    break;
                case TYPE_INDEX_BYTES:
                    dobj.setBytes((String)property, (byte [])value);
                    break;
                case TYPE_INDEX_DECIMAL:
                    dobj.setBigDecimal((String)property, (BigDecimal)value);
                    break;
                case TYPE_INDEX_INTEGER:   
                    dobj.setBigInteger((String)property, (BigInteger)value);
                    break;
                case TYPE_INDEX_DATE:  
                    dobj.setDate(((String)property), (Date)value);
                    break;
            }
        }
        // ***************************************
        // *** Use int based property setter 
        // ***************************************
        if( property instanceof Integer )
        {
            switch(type)
            {
                case TYPE_INDEX_BOOLEAN: 
                    dobj.setBoolean(((Integer)property).intValue(),((Boolean)value).booleanValue());
                    break;
                case TYPE_INDEX_BYTE:
                    dobj.setByte(((Integer)property).intValue(), ((Byte)value).byteValue());
                    break;
                case TYPE_INDEX_CHARACTER:
                    dobj.setChar(((Integer)property).intValue(), ((Character)value).charValue());
                    break;
                case TYPE_INDEX_DOUBLE: 
                    dobj.setDouble(((Integer)property).intValue(), ((Double)value).doubleValue());
                    break;
                case TYPE_INDEX_FLOAT:
                    dobj.setFloat((((Integer)property).intValue()), ((Float)value).floatValue());
                    break;
                case TYPE_INDEX_INT:  
                    dobj.setInt(((Integer)property).intValue(), ((Integer)value).intValue());
                    break;
                case TYPE_INDEX_LONG: 
                    dobj.setLong(((Integer)property).intValue(), ((Long)value).longValue());
                    break;
                case TYPE_INDEX_SHORT: 
                    dobj.setShort(((Integer)property).intValue(), ((Short)value).shortValue());
                    break;
                case TYPE_INDEX_STRING:
                    dobj.setString(((Integer)property).intValue(), (String)value);
                    break;
                case TYPE_INDEX_BYTES:
                    dobj.setBytes(((Integer)property).intValue(), (byte [])value);
                    break;
                case TYPE_INDEX_DECIMAL:
                    dobj.setBigDecimal(((Integer)property).intValue(), (BigDecimal)value);
                    break;
                case TYPE_INDEX_INTEGER:   
                    dobj.setBigInteger(((Integer)property).intValue(), (BigInteger)value);
                    break;
                case TYPE_INDEX_DATE:  
                    dobj.setDate((((Integer)property).intValue()), (Date)value);
                    break;
            }
        }
        // ***************************************
        // *** Use Property based property setter 
        // ***************************************
        if( property instanceof Property )
        {
            switch(type)
            {
                case TYPE_INDEX_BOOLEAN: 
                    dobj.setBoolean((Property)property,((Boolean)value).booleanValue());
                    break;
                case TYPE_INDEX_BYTE:
                    dobj.setByte((Property)property, ((Byte)value).byteValue());
                    break;
                case TYPE_INDEX_CHARACTER:
                    dobj.setChar((Property)property, ((Character)value).charValue());
                    break;
                case TYPE_INDEX_DOUBLE: 
                    dobj.setDouble((Property)property, ((Double)value).doubleValue());
                    break;
                case TYPE_INDEX_FLOAT:
                    dobj.setFloat(((Property)property), ((Float)value).floatValue());
                    break;
                case TYPE_INDEX_INT:  
                    dobj.setInt((Property)property, ((Integer)value).intValue());
                    break;
                case TYPE_INDEX_LONG: 
                    dobj.setLong((Property)property, ((Long)value).longValue());
                    break;
                case TYPE_INDEX_SHORT: 
                    dobj.setShort((Property)property, ((Short)value).shortValue());
                    break;
                case TYPE_INDEX_STRING:
                    dobj.setString((Property)property, (String)value);
                    break;
                case TYPE_INDEX_BYTES:
                    dobj.setBytes((Property)property, (byte [])value);
                    break;
                case TYPE_INDEX_DECIMAL:
                    dobj.setBigDecimal((Property)property, (BigDecimal)value);
                    break;
                case TYPE_INDEX_INTEGER:   
                    dobj.setBigInteger((Property)property, (BigInteger)value);
                    break;
                case TYPE_INDEX_DATE:  
                    dobj.setDate(((Property)property), (Date)value);
                    break;
            }
        }
    }
    
}

