/**
 *
 *  Licensed to the Apache Software Foundation (ASF) under one
 *  or more contributor license agreements.  See the NOTICE file
 *  distributed with this work for additional information
 *  regarding copyright ownership.  The ASF licenses this file
 *  to you under the Apache License, Version 2.0 (the
 *  "License"); you may not use this file except in compliance
 *  with the License.  You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an
 *  "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 *  KIND, either express or implied.  See the License for the
 *  specific language governing permissions and limitations
 *  under the License.
 */
package com.ibm.sdo.tck.tests.core;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.math.BigDecimal;
import java.net.URL;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;


import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;


import org.junit.Before;
import org.junit.Test;

import com.ibm.sdo.tck.framework.TCKTestBase;
import com.ibm.sdo.tck.util.XMLCompareUtil;

import commonj.sdo.DataObject;
import commonj.sdo.Property;
import commonj.sdo.Type;
import commonj.sdo.helper.DataFactory;
import commonj.sdo.helper.HelperContext;
import commonj.sdo.helper.TypeHelper;
import commonj.sdo.helper.XMLHelper;
import commonj.sdo.helper.XSDHelper;

public class OpenTypeTest extends TCKTestBase {
    private HelperContext hc;
    private TypeHelper th;
    private XSDHelper xsdh;
    private XMLHelper xmlh;
    private DataFactory df;
    private final String TEST_MODEL = "/sharedFiles/open.xsd";
    private final String TEST_NAMESPACE = "http://www.example.com/open";

    /**
     * Open content SDO 2 test.
     */
    @Test
    public void testOpenType() throws IOException {
        Type quoteType = th.getType(TEST_NAMESPACE, "OpenQuote");
        DataObject quote = df.create(quoteType);

        quote.setString("symbol", "s1");
        quote.setDouble("volume", (double)1000.1);

        //System.out.println("Instance properties size 1: "
        //                  + quote.getInstanceProperties().size());
        for(Iterator iter = quote.getInstanceProperties().iterator(); iter
           .hasNext();) {
            Property property = (Property) iter.next();
            //System.out.println("  property: " + property.getName());
        }

        Property companyProperty = xsdh.getGlobalProperty(TEST_NAMESPACE,
                                                          "company", true);
        DataObject company = quote.createDataObject(companyProperty);
        company.setString("name", "FlyByNightTechnology");

        Property priceProperty = xsdh.getGlobalProperty(TEST_NAMESPACE,
                                                        "price", true);
        // quote.getList(priceProperty).add(new BigDecimal("1000.9"));
        List<BigDecimal> tempList = new ArrayList<BigDecimal>();
        tempList.add(new BigDecimal("1000.9"));
        quote.setList(priceProperty, tempList);

        //System.out.println("Instance properties size 2: "
        //              + quote.getInstanceProperties().size());
        for(Iterator iter = quote.getInstanceProperties().iterator(); iter
           .hasNext();) {
            Property property = (Property) iter.next();
            //System.out.println("  property: " + property.getName());
        }

        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        xmlh.save(quote, TEST_NAMESPACE, "openStockQuote", baos);

        //System.out.println(baos.toString());

        URL url = getClass().getResource("/OpenTypeTest/openQuote2.xml");
        InputStream inputStream = url.openStream();
        assertTrue(XMLCompareUtil.compareXMLSimilar(inputStream, baos.toString()));
        inputStream.close();

        // assertTrue(TestUtil.equalXmlFiles(new
        // ByteArrayInputStream(baos.toByteArray()),
        // getClass().getResource(TEST_DATA)));
    }

    @Test
    public void getManyValuesTest()
    {
        Type quoteType = th.getType(TEST_NAMESPACE, "OpenQuote");
        DataObject quote = df.create(quoteType);

        // Try to retrieve an unknown property, verify the return value is null.
        List l = (List)quote.get("somePropertyName");
        assertNull(l);
    }
    
    @Before
    public void setUp() throws Exception {
        super.setUp();

        // Populate the meta data for the test (Stock Quote) model
        URL url = getClass().getResource(TEST_MODEL);
        InputStream inputStream = url.openStream();
        hc = getScope();
        th = hc.getTypeHelper();
        xsdh = hc.getXSDHelper();
        df = hc.getDataFactory();
        xmlh = hc.getXMLHelper();
        hc.getXSDHelper().define(inputStream, url.toString());
        inputStream.close();
    }
}

