/*
 * <copyright>
 * 
 * SDOPathTest.java
 *
 * Licensed Materials - Property of IBM
 * Restricted Materials of IBM
 * 
 * (c) Copyright IBM Corporation 2009.  All Rights Reserved. 
 * Note to U.S. Government Users Restricted Rights:  Use, duplication or  
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp. 
 * 
 * </copyright>
 * 
 */
package com.ibm.sdo.tck.tests.core;

import static org.junit.Assert.*;

import java.io.IOException;
import java.io.InputStream;
import java.net.URL;


import junit.framework.Assert;

import org.junit.Test;

import com.ibm.sdo.tck.testcasebase.PathTestBase;

import commonj.sdo.DataObject;

public class SDOPathTest extends PathTestBase {
	
    /**
	 * Test @property only binds to attribute property
	 * 
	 * @throws IOException
	 */
	@Test
	public void sdoPathTestAttribute() throws IOException
	{
		DataObject bookstore = xmlHelper.load(getClass().getResourceAsStream(RES_PATH_BOOKSTOREXML)).getRootObject();
		DataObject book = (DataObject)bookstore.getList("book").get(0);
		//System.out.println(book.get("category"));
		//System.out.println(book.get("@category"));
		assertEquals(book.get("category"), book.get("@category"));
	}
	
	/**
	 * Test DataObject.get() on unset property with default value
	 * 
	 * @throws IOException
	 */
	@Test
	public void sdoPathDefaultValue() throws IOException
	{
		DataObject bookstore = xmlHelper.load(getClass().getResourceAsStream(RES_PATH_BOOKSTOREXML)).getRootObject();
		DataObject book = (DataObject)bookstore.getList("book").get(0);
		assertFalse(book.isSet("year"));
		assertEquals(2000, book.getInt("year"));
		
		DataObject title = book.getDataObject("title");
		assertFalse(title.isSet("@lang"));
		assertEquals("fr", title.get("@lang"));
	}
	
	/**
	 * Test invalid path container
	 * 
	 * @throws IOException
	 */
	@Test
	public void sdoPathInvalidContainerTest() throws IOException {
		URL url = getClass().getResource(RES_PATH_BOOKSTOREXML);
		InputStream is = url.openStream();
		DataObject root = xmlHelper.load(is).getRootObject(); // Point to
		// <bookstore>
		is.close();
		Object title = root.get("book[]/title");
		Assert.assertNull(title);
	}

	/**
	 * Test the path container with a constraint
	 * 
	 * @throws IOException
	 */
	@Test
	public void sdoPathContainerTest() throws IOException {
		URL url = getClass().getResource(RES_PATH_BOOKSTOREXML);
		InputStream is = url.openStream();
		DataObject root = xmlHelper.load(is).getRootObject(); // Point to
		// <bookstore>
		// is.close();
		DataObject title = (DataObject) root.get("book[category='WEB']/title");
		Assert.assertEquals("XQuery Kick Start", title.getString("value"));
	}

	/**
	 * Test the last step with a constraint
	 * 
	 * @throws IOException
	 */
	@Test
	public void sdoPathTest() throws IOException {
		URL url = getClass().getResource(RES_PATH_BOOKSTOREXML);
		InputStream is = url.openStream();
		DataObject root = xmlHelper.load(is).getRootObject(); // Point to
		// <bookstore>

		DataObject obj = (DataObject)root.get("book[category='WEB']");
        DataObject title = (DataObject)obj.get("title");
        Assert.assertEquals("XQuery Kick Start", title.getString("value"));

		String sdoPath = "book[3]/author[2]";
		Assert.assertEquals("Per Bothner", root.getString(sdoPath));
		root.unset(sdoPath);
		// Make sure the next author took the new index position
		Assert.assertEquals("Kurt Cagle", root.getString(sdoPath));

		/* Per the JSR 235 expert group decision, SDO path access
		 * using a '.' is implementation specific and will not be
		 * tested at this time.
		 */
		// Test the SDO path name containing dot (.)
		/*
		sdoPath = "book.0/subject.2";
		Assert.assertEquals("International Cooking", root.get(sdoPath));
		Assert.assertTrue(root.isSet(sdoPath));
		root.unset(sdoPath);
		Assert.assertFalse(root.isSet(sdoPath));
		Assert.assertNull(root.get(sdoPath));
		root.set(sdoPath, "Italian Cooking");
		Assert.assertEquals("Italian Cooking", root.get(sdoPath));
		*/
	}

	/**
	 * Test DataObject.set(), setXXXX(), isSet(), and unset() with [index] in
	 * the expression
	 * 
	 * @throws IOException
	 */
	@Test
	public void sdoPathTest_set_bracketIndex() throws IOException {
		bookstoreTest_set_bracketIndex("book[3]/author[2]");
	}

	/**
	 * Test DataObject.unset() on many-value property
	 * 
	 * @throws IOException
	 */
	@Test
	public void sdoPathTest_unset_many() throws IOException {
		URL url = getClass().getResource(RES_PATH_BOOKSTOREXML);
		InputStream is = url.openStream();
		DataObject root = xmlHelper.load(is).getRootObject(); // Point to
		// <bookstore>

		DataObject book = (DataObject) root.get("book[category='WEB']");
		book.unset("author");
		Assert.assertFalse(book.isSet("author"));
	}
	
	/**
	 * Test get(), set(), getString(), and setString() using SDO path on xsd:QName property
	 */
	@Test
	public void sdoPathXSDQNameTest() {
		String sdoPath4test = "policy";

		xsdHelper.define(xsdString);
		DataObject root = xmlHelper.load(xmlString).getRootObject();
		Object obj = root.get(sdoPath4test);
		assertTrue(obj instanceof String);
		assertEquals("http://www.example.com/simple#stockQuote", obj);
		
		root.set(sdoPath4test, "http://www.example.com/simple#stockQuote2");
		assertEquals("http://www.example.com/simple#stockQuote2", root.getString(sdoPath4test));
		
		root.setString(sdoPath4test, "http://www.example.com/simple#stockQuote");
		assertEquals("http://www.example.com/simple#stockQuote", root.get(sdoPath4test));
	}
	
}
