/*
 * <copyright>
 * 
 * AnnotationsDynamicMetadataTest.java
 *
 * Licensed Materials - Property of IBM
 * Restricted Materials of IBM
 * 
 * (c) Copyright IBM Corporation 2009.  All Rights Reserved. 
 * Note to U.S. Government Users Restricted Rights:  Use, duplication or  
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp. 
 * 
 * </copyright>
 * 
 */
package com.ibm.sdo.tck.tests.dynamictypes;

import java.util.ArrayList;
import java.util.List;


import org.junit.Before;

import com.ibm.sdo.tck.testcasebase.AnnotationsTestBase;

import commonj.sdo.DataObject;
import commonj.sdo.Type;

public class AnnotationsDynamicMetadataTest extends AnnotationsTestBase
{
    Type stringType;
    Type intType;
    
    @Before
    public void setUp() throws Exception 
    {
        super.setUp();
        
        stringType = scope.getTypeHelper().getType("commonj.sdo", "String");
        intType = scope.getTypeHelper().getType("commonj.sdo", "Int");

        defineSequenceAnnotationType();
        defineAliasAnnotationType();
        defineReadOnlyAnnotationType();
        defineManyAnnotationType();
        defineOppositePropertyType();
    }
    
    private void defineSequenceAnnotationType()
    {        
        DataObject typeDO = scope.getDataFactory().create("commonj.sdo", "Type");
        typeDO.set("uri", NS);
        typeDO.set("name", "seqfoo1");
        typeDO.set("sequenced", Boolean.TRUE);
        
        for( int i=1; i<=3; i++ )
        {
            DataObject prop = typeDO.createDataObject("property");
            prop.set("name", "seqbar"+i);
            prop.set("type", stringType);
        }            
        
        scope.getTypeHelper().define(typeDO);
    }
    
    private void defineReadOnlyAnnotationType()
    {        
        DataObject typeDO = scope.getDataFactory().create("commonj.sdo", "Type");
        typeDO.set("uri", NS);
        typeDO.set("name", "rofoo1");
        
        DataObject prop = typeDO.createDataObject("property");
        prop.set("name", "robar1");
        prop.set("type", stringType);
        prop.set("readOnly", Boolean.FALSE);
        
        prop = typeDO.createDataObject("property");
        prop.set("name", "robar2");
        prop.set("type", stringType);
        prop.set("readOnly", Boolean.TRUE);
        
        prop = typeDO.createDataObject("property");
        prop.set("name", "robar3");
        prop.set("type", stringType);
        
        scope.getTypeHelper().define(typeDO);
    }
    
    private void defineManyAnnotationType()
    {        
        DataObject typeDO = scope.getDataFactory().create("commonj.sdo", "Type");
        typeDO.set("uri", NS);
        typeDO.set("name", "manyfoo1");
        
        DataObject prop = typeDO.createDataObject("property");
        prop.set("name", "manybar1");
        prop.set("type", stringType);
        prop.set("many", Boolean.FALSE);
        
        prop = typeDO.createDataObject("property");
        prop.set("name", "manybar2");
        prop.set("type", stringType);
        prop.set("many", Boolean.TRUE);
        
        prop = typeDO.createDataObject("property");
        prop.set("name", "manybar3");
        prop.set("type", stringType);
        
        scope.getTypeHelper().define(typeDO);
    }
    
    private void defineAliasAnnotationType()
    {        
        DataObject typeDO = scope.getDataFactory().create("commonj.sdo", "Type");
        typeDO.set("uri", NS);
        typeDO.set("name", "aliasfoo1");
        List aliases = new ArrayList();
        aliases.add("aliasfoo2");        
        typeDO.set("aliasName", aliases);
        
        DataObject prop = typeDO.createDataObject("property");
        prop.set("name", "aliasbar1");
        prop.set("type", stringType);
        aliases = new ArrayList();
        aliases.add("aliasbar7"); 
        aliases.add("aliasbar8"); 
        aliases.add("aliasbar9"); 
        prop.set("aliasName", aliases);
        
        prop = typeDO.createDataObject("property");
        prop.set("name", "aliasbar2");
        prop.set("type", stringType);
        
        try
        {
            scope.getTypeHelper().define(typeDO);
        }
        catch(UnsupportedOperationException e)
        {
            // Making an allowance, for now due to some impl limitations.
        }
    }
    
    private void defineOppositePropertyType()
    {   
        DataObject reffoo1 = scope.getDataFactory().create("commonj.sdo", "Type");
        reffoo1.set("uri", NS);
        reffoo1.set("name", "reffoo1");

        DataObject reffoo2 = scope.getDataFactory().create("commonj.sdo", "Type");
        reffoo2.set("uri", NS);
        reffoo2.set("name", "reffoo2");
        
        DataObject refbar1  = reffoo1.createDataObject("property");
        refbar1.set("name", "refbar1");
        refbar1.set("type", reffoo2);
        
        DataObject refbar2  = reffoo2.createDataObject("property");
        refbar2.set("name", "refbar2");
        refbar2.set("type", reffoo1);

        refbar1.setDataObject("opposite", refbar2);
        refbar2.setDataObject("opposite", refbar1);
        
        List l = new ArrayList();
        l.add(reffoo1);
        l.add(reffoo2);
        
        scope.getTypeHelper().define(l);
    }
}
