/*
 *  Licensed to the Apache Software Foundation (ASF) under one
 *  or more contributor license agreements.  See the NOTICE file
 *  distributed with this work for additional information
 *  regarding copyright ownership.  The ASF licenses this file
 *  to you under the Apache License, Version 2.0 (the
 *  "License"); you may not use this file except in compliance
 *  with the License.  You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an
 *  "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 *  KIND, either express or implied.  See the License for the
 *  specific language governing permissions and limitations
 *  under the License.
 *  
 */
package com.ibm.sdo.tck.tests.helpers;


import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;


import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import com.ibm.sdo.tck.framework.TCKTestBase;
import com.ibm.sdo.tck.framework.annotations.Operation;
import com.ibm.sdo.tck.framework.annotations.SpecRef;
import com.ibm.sdo.tck.framework.annotations.TCKTest;


import commonj.sdo.helper.HelperContext;
import commonj.sdo.impl.HelperProvider;

// The following empty meta-data tag is currently required in order for
// post-jar processing of the meta-data tags above methods below.  
@TCKTest()
public class HelperContextTest extends TCKTestBase
{
    protected HelperContext helperContext;
    
    @Before
    public void setUp () throws Exception {
    	super.setUp();
    	helperContext = HelperProvider.getDefaultContext();
    }
    
    @After
    public void tearDown() throws Exception {
      super.tearDown();
    }

    /**
     * Verifies the HelperProvider.getDefaultContext method. <p/>
     * @see commonj.sdo.HelperProvider#getDefaultContext()
     * @see <a href="http://osoa.org/download/attachments/36/Java-SDO-Spec-v2.1.0-FINAL.pdf?version=1#page=54">2.1 spec section 3.16.1</a>
     */
    @TCKTest(specRef={@SpecRef(version="2.1",section = "3.16.1")},
            testFor={@Operation(className="HelperProvider",methodName="getDefaultContext",signature="")})    
    @Test
    public void testGetDefaultContext() {
        validateHelper(helperContext, 
                       commonj.sdo.helper.HelperContext.class);
    }
      
    /**
     * Verifies the HelperContext.getCopyHelper method. <p/>
     * @see commonj.sdo.HelperContext#getCopyHelper()
     * @see <a href="http://osoa.org/download/attachments/36/Java-SDO-Spec-v2.1.0-FINAL.pdf?version=1#page=54">2.1 spec section 3.16.1</a>
     */
    @TCKTest(specRef={@SpecRef(version="2.1",section = "3.16.1")},
            testFor={@Operation(className="HelperContext",methodName="getCopyHelper",signature="")})    
    @Test
    public void testGetCopyHelper() {
        validateHelper(helperContext.getCopyHelper(), 
                       commonj.sdo.helper.CopyHelper.class);
    }
    
    /**
     * Verifies the HelperContext.getDataFactory method. <p/>
     * @see commonj.sdo.HelperContext#getDataFactory()
     * @see <a href="http://osoa.org/download/attachments/36/Java-SDO-Spec-v2.1.0-FINAL.pdf?version=1#page=54">2.1 spec section 3.16.1</a>
     */
    @TCKTest(specRef={@SpecRef(version="2.1",section = "3.16.1")},
            testFor={@Operation(className="HelperContext",methodName="getDataFactory",signature="")})    
    @Test
    public void testGetDataFactory() {
        validateHelper(helperContext.getDataFactory(), 
                       commonj.sdo.helper.DataFactory.class);
    }
    
    /**
     * Verifies the HelperContext.getDataHelper method. <p/>
     * @see commonj.sdo.HelperContext#getDataHelper()
     * @see <a href="http://osoa.org/download/attachments/36/Java-SDO-Spec-v2.1.0-FINAL.pdf?version=1#page=54">2.1 spec section 3.16.1</a>
     */
    @TCKTest(specRef={@SpecRef(version="2.1",section = "3.16.1")},
            testFor={@Operation(className="HelperContext",methodName="getDataHelper",signature="")})    
    @Test
    public void testGetDataHelper() {
        validateHelper(helperContext.getDataHelper(), 
                       commonj.sdo.helper.DataHelper.class);
    }
    
    /**
     * Verifies the HelperContext.getEqualityHelper method. <p/>
     * @see commonj.sdo.HelperContext#getEqualiltyHelper()
     * @see <a href="http://osoa.org/download/attachments/36/Java-SDO-Spec-v2.1.0-FINAL.pdf?version=1#page=54">2.1 spec section 3.16.1</a>
     */
    @TCKTest(specRef={@SpecRef(version="2.1",section = "3.16.1")},
            testFor={@Operation(className="HelperContext",methodName="getEqualityHelper",signature="")})    
    @Test
    public void testGetEqualityHelper() {
        validateHelper(helperContext.getEqualityHelper(), 
                       commonj.sdo.helper.EqualityHelper.class);
    }
    
    /**
     * Verifies the HelperContext.getTypeHelper method. <p/>
     * @see commonj.sdo.HelperContext#getTypeHelper()
     * @see <a href="http://osoa.org/download/attachments/36/Java-SDO-Spec-v2.1.0-FINAL.pdf?version=1#page=54">2.1 spec section 3.16.1</a>
     */
    @TCKTest(specRef={@SpecRef(version="2.1",section = "3.16.1")},
            testFor={@Operation(className="HelperContext",methodName="getTypeHelper",signature="")})    
    @Test
    public void testGetTypeHelper() {
        validateHelper(helperContext.getTypeHelper(), 
                       commonj.sdo.helper.TypeHelper.class);
    }
    
    /**
     * Verifies the HelperContext.getXMLHelper method. <p/>
     * @see commonj.sdo.HelperContext#getXMLHelper()
     * @see <a href="http://osoa.org/download/attachments/36/Java-SDO-Spec-v2.1.0-FINAL.pdf?version=1#page=54">2.1 spec section 3.16.1</a>
     */
    @TCKTest(specRef={@SpecRef(version="2.1",section = "3.16.1")},
            testFor={@Operation(className="HelperContext",methodName="getXMLHelper",signature="")})    
    @Test
    public void testGetXMLHelper() {
        validateHelper(helperContext.getXMLHelper(), 
                       commonj.sdo.helper.XMLHelper.class);
    }
    
    /**
     * Verifies the HelperContext.getXSDHelper method. <p/>
     * @see commonj.sdo.HelperContext#getXSDHelper()
     * @see <a href="http://osoa.org/download/attachments/36/Java-SDO-Spec-v2.1.0-FINAL.pdf?version=1#page=54">2.1 spec section 3.16.1</a>
     */
    @TCKTest(specRef={@SpecRef(version="2.1",section = "3.16.1")},
            testFor={@Operation(className="HelperContext",methodName="getXSDHelper",signature="")})    
    @Test
    public void testGetXSDHelper() {
        validateHelper(helperContext.getXSDHelper(), 
                       commonj.sdo.helper.XSDHelper.class);
    }
    
    /*
     * Common method for validating helpers
     */
    private void validateHelper(Object helper, Class expectedInterface )
    {
        assertNotNull(helper);
        assertTrue(expectedInterface.isInstance(helper));
    }
    
 
}
