/*
 *  Licensed to the Apache Software Foundation (ASF) under one
 *  or more contributor license agreements.  See the NOTICE file
 *  distributed with this work for additional information
 *  regarding copyright ownership.  The ASF licenses this file
 *  to you under the Apache License, Version 2.0 (the
 *  "License"); you may not use this file except in compliance
 *  with the License.  You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an
 *  "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 *  KIND, either express or implied.  See the License for the
 *  specific language governing permissions and limitations
 *  under the License.
 *  
 *  $Rev: 537795 $  $Date: 2008/02/06 15:34:14 $
 */
package com.ibm.sdo.tck.tests.metadata;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.io.InputStream;
import java.net.URL;
import java.util.List;


import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import com.ibm.sdo.tck.framework.TCKTestBase;
import com.ibm.sdo.tck.framework.annotations.Operation;
import com.ibm.sdo.tck.framework.annotations.TCKTest;
import com.ibm.sdo.tck.util.TCKUtil;


import commonj.sdo.DataObject;
import commonj.sdo.Property;
import commonj.sdo.Type;
import commonj.sdo.helper.HelperContext;
import commonj.sdo.helper.TypeHelper;
import commonj.sdo.helper.XMLDocument;

@TCKTest(testFor = {@Operation(className="DataObject", methodName="unset", signature="int"),
                    @Operation(className="DataObject", methodName="getInstanceProperties", signature=""),
                    @Operation(className="DataObject", methodName="getInstanceProperty", signature="String"),
                    @Operation(className="DataObject", methodName="get", signature="Property"),
                    @Operation(className="Property", methodName="getDefault", signature=""),                    
                    @Operation(className="Property", methodName="getAliasNames", signature=""),
                    @Operation(className="Property", methodName="getContainingType", signature=""),
                    @Operation(className="Property", methodName="getName", signature=""),
                    @Operation(className="Property", methodName="getType", signature=""),
                    @Operation(className="Property", methodName="isContainment", signature=""),
                    @Operation(className="Property", methodName="isMany", signature=""),
                    @Operation(className="Property", methodName="isReadOnly", signature=""),
                    @Operation(className="Property", methodName="get", signature="Property"),
                    @Operation(className="Property", methodName="getInstanceProperties", signature=""),
                    @Operation(className="Property", methodName="getOpposite", signature="", postcondition="opposite property is null")})
public class PropertyTest extends TCKTestBase {
	  private DataObject testDO;
    

    @Before
    public void setUp () throws Exception {
      super.setUp();
      TCKUtil.defineXSDAPITestTypes(getScope());
      testDO = TCKUtil.createTestDataObject(getScope(), TCKUtil.API_TYPE);
    }
    
    
    @After
    public void tearDown() throws Exception {
      super.tearDown();
    }

    /**
     * Verify Property.getDefault() for a newly created DataObject.
     * @throws Exception 
     */
    @Test
    public void testGetDefaultNewDO() throws Exception {
        verifyGetDefault(TCKUtil.createTestDataObject(getScope(), TCKUtil.API_TYPE));
    }

    /**
     * Verify Property.getDefault() for a cleared DataObject.
     */
    @Test
    public void testGetDefaultClearedDO() {
        // Set the values for each Property using populateFields, then unset
        // them using unsetFields.
        try {
            TCKUtil.populateAPITestObjectProperties(testDO, getScope());
        } catch (Exception e) {
            fail("could not populate DataObject fields");
        }
        unsetFields(testDO);
        verifyGetDefault(testDO);
    }

    /**
     * Verify Property.getName()
     */
    @Test
    public void testPropertyGetName() {
        Property property = testDO.getInstanceProperty("containMany");
        assertEquals("Property.getName() returned an unexpected value.", property.getName(), "containMany");
    }

    /**
     * Verify Property.getType()
     */
    @Test
    public void testPropertyGetType() {
        Property property = testDO.getInstanceProperty("containMany");

        assertEquals("Property.getType() returned an unexpected value.", property.getType(), testDO.getType());
    }

    /**
     * Verify Property.isMany()==true
     */
    @Test
    public void testPropertyIsManyTrue() {
        Property property = testDO.getInstanceProperty("containMany");
        assertTrue("Property.isMany() returned an unexpected value.", property.isMany());
    }

    /**
     * Verify Property.isMany()==false
     */
    @Test
    public void testPropertyIsManyFalse() {
        Property property = testDO.getInstanceProperty("booleanVal");
        assertFalse("Property.isMany() returned an unexpected value.", property.isMany());
    }

    /**
     * Verify Property.isContainment()==true
     */
    @Test
    public void testPropertyIsContainmentTrue() {
        Property property = testDO.getInstanceProperty("containMany");
        assertTrue("Property.isContainment() returned an unexpected value.", property.isContainment());
    }

    /**
     * Verify Property.isContainment()==false
     */
    @Test
    public void testPropertyIsContainmentFalse() {
        Property property = testDO.getInstanceProperty("booleanVal");
        assertFalse("Property.isContainment() returned true when expecting false.", property.isContainment());
    }

    /**
     * Verify Property.isReadOnly()==true
     */
    @Test
    public void testPropertyIsReadOnlyTrue() {
    	Property property = testDO.getInstanceProperty("readOnlyVal");
        assertTrue("Property.isReadOnly() returned false when expecting true.", property.isReadOnly());
    }

    /**
     * Verify Property.isReadOnly()==false
     */
    @Test
    public void testPropertyIsReadOnlyFalse() {
        Property property = testDO.getInstanceProperty("booleanVal");
        assertFalse("Property.isReadOnly() returned true when expecting false.", property.isReadOnly());
    }

    /**
     * Verify Property.getContainingType
     */
    @Test
    public void testPropertyGetContainingType() {
        Property property = testDO.getInstanceProperty("booleanVal");
        assertEquals("Property.getContainingType() returned an unexpected value.", property.getContainingType(), testDO
            .getType());
    }

    /**
     * Verify Property.getAliasNames() == empty List
     */
    @Test
    public void testPropertyGetAliasNamesEmpty() {
        Property property = testDO.getInstanceProperty("booleanVal");
        assertEquals("Property.getAliasNames() returned a List of unexpected size.", 0, property.getAliasNames().size());
    }

    /**
     * Verify Property.getAliasNames() == populated List
     */
    @Test
    public void testPropertyGetAliasNames() {
        List aliasNames = testDO.getInstanceProperty("decimalVal2").getAliasNames();
        assertEquals("Property.getAliasNames() returned a List of unexpected size.", 1, aliasNames.size());

        String alias = (String)aliasNames.get(0);
        assertEquals("Property.getAliasNames() returned a List with unexpected contents.", "dec2", alias);
    }

    /**
     * Verify Property.getOpposite() == null
     */
    @Test
    public void testPropertyGetOppositeNull() {
        Property property = testDO.getInstanceProperty("booleanVal");
        assertNull("Property.getOpposite() did not return null as expected.", property.getOpposite());
    }

    /**
     * Verify Property.getOpposite() != null
     * @throws Exception 
     */
    @Test
    @TCKTest(
        testFor={@Operation(className="Property",methodName="getOpposite")}
    )
    public void testPropertyGetOpposite() throws Exception {
        final URL url = getClass().getResource("/PropertyTest/sports.xsd");
        final InputStream inputStream = url.openStream();
        getScope().getXSDHelper().define(inputStream, url.toString());
        inputStream.close();
        
        final URL url2 = getClass().getResource("/PropertyTest/sports.xml");
        final InputStream inputStream2 = url2.openStream();
        XMLDocument unionDoc = getScope().getXMLHelper().load(inputStream2, url2.toString(), null);
        inputStream2.close();
        
        DataObject union = unionDoc.getRootObject();
        DataObject p0 = union.getDataObject("member[ID='p0']");
        DataObject p3 = union.getDataObject("member[ID='p3']");
        DataObject t0 = union.getDataObject("team[ID='t0']");
        
        // validate property
        Property prop1  = t0.getType().getProperty("captain");
        Property prop2  = p0.getType().getProperty("captainFor");
        assertEquals(prop2,prop1.getOpposite());
        assertEquals(prop1,prop2.getOpposite());
        
        // validate instance         
        assertTrue( "p 0".equals(((DataObject)t0.get("captain")).getString("name")));
        assertTrue( "t 0".equals(((DataObject)p0.get("captainFor")).getString("name")));
        assertNull( p3.get("captainFor") );
        
        // Make change
        t0.set("captain", p3);

        // validate changed instance 
        assertTrue( "p 3".equals(((DataObject)t0.get("captain")).getString("name")));
        assertTrue( "t 0".equals(((DataObject)p3.get("captainFor")).getString("name")));
        assertNull( p0.get("captainFor") );
    }

    /**
     * Verify instance properties
     */
    @Test
    public void testInstanceProperties() {
        String NS       = "http://example.com/simple";
        String Name     = "Simple";
        
        HelperContext scope = getScope();
        TypeHelper typeHelper = scope.getTypeHelper();
        Type StringType = typeHelper.getType("commonj.sdo", "String");
        DataObject type = scope.getDataFactory().create("commonj.sdo", "Type");
        type.set("uri", NS);
        type.set("name", Name);
        DataObject StringProperty = type.createDataObject("property");
        StringProperty.set("name", "StringVal");
        StringProperty.set("type", StringType);
        
        //construct two open content properties
        DataObject OpenPropertyDO = scope.getDataFactory().create("commonj.sdo","Property");
        OpenPropertyDO.set("name", "Open");
        OpenPropertyDO.set("type", StringType);
        Property OpenProperty = typeHelper.defineOpenContentProperty(NS, OpenPropertyDO);
        StringProperty.set(OpenProperty, "openstring");
        
        Property openContentProperty = typeHelper.getOpenContentProperty("commonj.sdo/xml", "xmlElement");
        StringProperty.set(openContentProperty, Boolean.TRUE);
        
        typeHelper.define(type);
        Type t = scope.getTypeHelper().getType(NS,Name);
        Property p = t.getProperty("StringVal");
        
        Property openProp = TCKUtil.getInstanceProperty(p, "Open");
        assertNotNull(openProp);
        assertTrue("openstring".equals((String)p.get(openProp)));
        openProp = TCKUtil.getInstanceProperty(p, "xmlElement");
        assertNotNull(openProp);
        assertTrue(((Boolean)p.get(openProp)).booleanValue());
    }
    
    
    /**
     * verifyGetDefault is a private method that loops through each Property in
     * the passed DataObject and verifies that the value is equal to the value
     * returned by getDefault for each Property. The precondition is that unset
     * has been called for any Property for which a value has been set.
     * 
     * @param testedDO
     */

    private void verifyGetDefault(DataObject testedDO) 
    {
        List properties = testedDO.getInstanceProperties();
        Property property;

        for (int i = 0; i < properties.size(); i++) 
        {
            property = (Property)properties.get(i);
            if (!(property.isReadOnly())) 
            {
                if (property.isMany()) 
                {
                    assertNull("Property.getDefault() did not return null for many-valued Property " + property.getName()
                        + "'.", property.getDefault());
                }
                else if ("Bytes".equals(property.getType().getName()) )
                {
                    byte [] v1 =  testedDO.getBytes(property);
                    byte [] v2 =  (byte [])property.getDefault();
                        
                    if( v1 == null || v2 == null )
                    {
                        if( v1 != v2 )
                        {
                            assertEquals("Property.getDefault() did not return the correct default value for Property '" + property
                                    .getName()
                                    + "'.", v1, v2);
                        }
                    }
                    else
                    {
                        assertEquals("Property.getDefault() did not return the correct default value for Property '" + property
                                .getName()
                                + "'.", v1.length, v2.length);
                        for( int ii=0; ii<v1.length; ii++ )
                        {
                             assertEquals("Property.getDefault() did not return the correct default value for Property '" + property
                                     .getName()
                                     + "'.", v1[ii], v2[ii]);
                        }
                    }
                }
                else 
                {
 
                    assertEquals("Property.getDefault() did not return the correct default value for Property '" + property
                        .getName()
                        + "'.", testedDO.get(property), property.getDefault());
                }
            }
        }
    }

    /**
     * unsetFields() is a private method that loops through the Properties in
     * the passed DataObject and unsets each Property.
     * 
     * @param unsetDO
     */

    private void unsetFields(DataObject unsetDO) {
        List properties = unsetDO.getInstanceProperties();
        Property currProp;

        for (int i = 0; i < properties.size(); i++)
        {
        	currProp = (Property) properties.get(i);
        	if (!(currProp.isReadOnly())) {
               unsetDO.unset(i);
        	}
        }
    }
    
    /**
     * Verify simple default values
     */
    @Test
    public void testDefaultAndFixedSimpleValues() {
        String NS   = "http://www.example.com/tck_api_test";
        String TYPE = "SubAPITest";
        
        DataObject dobj = getScope().getDataFactory().create(NS,TYPE);
        assertNotNull(dobj);
        
        // Test Elements
        assertNull(dobj.getString("subString1"));
        assertEquals("sub2",dobj.getString("subString2"));
        assertEquals("sub3",dobj.getString("subString3"));
        
        // Test Attributes
        assertNull(dobj.getString("subString4"));
        assertEquals("sub5",dobj.getString("subString5"));
        assertEquals("sub6",dobj.getString("subString6"));
        
    }


}
