/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.    
 */

package com.ibm.sdo.tck.tests.xml;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

import java.io.IOException;
import java.io.InputStream;
import java.math.BigDecimal;
import java.net.URL;
import java.util.Iterator;
import java.util.List;


import org.junit.Before;
import org.junit.Test;

import com.ibm.sdo.tck.framework.TCKTestBase;
import com.ibm.sdo.tck.util.EqualityTest;
import com.ibm.sdo.tck.util.XMLCompareUtil;

import commonj.sdo.DataObject;
import commonj.sdo.Property;
import commonj.sdo.Type;
import commonj.sdo.helper.HelperContext;
import commonj.sdo.helper.XMLDocument;

public class DefaultSerializationTest  extends TCKTestBase 
{
    static final String NS              = "http://www.example.com/dstest";
    static final String TYPE_NAME       = "DefaultSerialization";
    static final String ELEMENT_NAME    = "defaultSerialization";
    
    private HelperContext scope;
    protected static EqualityTest.GeneralComparator COMPARE_ANY = new EqualityTest.GeneralComparator();
   
    @Test
    public void testDefaultSerialization() throws IOException
    {
        byte [] defBytes = {0};
        Object [][] defaultValues = { {"sdoChar", '0'},
                                      {"stringVal",""},
                                      {"booleanVal",false},
                                      {"byteVal",(byte)0},
                                      {"decimalVal",new BigDecimal(0)},
                                      {"intVal",0},
                                      {"floatVal",(float)0}, 
                                      {"doubleVal",(double)0}, 
                                      {"shortVal",(short)0}, 
                                      {"longVal",(long)0}, 
                                      {"bytesVal", defBytes}, 
                                      {"integerVal", 0}};
        
        Type type = scope.getTypeHelper().getType(NS,TYPE_NAME);
        DataObject dobj = getScope().getDataFactory().create(type);
        
        // First test serialization of Character typed property
        dobj.set("sdoChar", dobj.get("sdoChar"));
        assertEquals((char)0, dobj.getChar("sdoChar"));
        
        String xml = scope.getXMLHelper().save(dobj, NS, ELEMENT_NAME);
        URL url = getClass().getResource("/DefaultSerializationTest/defaultSerialization.xml");
        InputStream inputStream = url.openStream();
        assertTrue(XMLCompareUtil.compareXML(inputStream, xml));
        inputStream.close();
        
        // Validate setChar works as well
        dobj.setChar("sdoChar", '0');
        assertEquals('0', dobj.getChar("sdoChar"));
        
        // Validate all simple types intrinsic default values
        List l = type.getProperties();
        Iterator itr = l.iterator();
        while( itr.hasNext() )
        {
            Property p = (Property)itr.next();
            if( p.getType().isDataType() && !p.isMany() && !p.isReadOnly() )
                dobj.set(p, dobj.get(p));
        }
        
        for( int i=0; i<defaultValues.length; i++ )
        {
            Object value = dobj.get((String)defaultValues[i][0]);
            assertEquals("Error comparing "+defaultValues[i][0],0, COMPARE_ANY.compare(value,defaultValues[i][1]));
        }
        
        xml = scope.getXMLHelper().save(dobj, NS, ELEMENT_NAME);
        //System.out.println(xml);
        XMLDocument testXMLDocument = scope.getXMLHelper().load(xml);
        assertNotNull("Deserialization returned a null value.",testXMLDocument);
        DataObject serDobj = testXMLDocument.getRootObject();
        EqualityTest et = new EqualityTest(serDobj,dobj,true);
        assertTrue("Serialization and deserialization resulted in a nonequivalent DataObject: " + et.getFailure(),
                   et.getResult() );        
    }
    
    @Before
    public void setUp() throws Exception 
    {
        super.setUp();

        scope = getScope();
        
        // Populate the meta data for the test (Stock Quote) model
        URL url = getClass().getResource("/sharedFiles/defaultSerialization.xsd");
        InputStream inputStream = url.openStream();
        scope.getXSDHelper().define(inputStream, url.toString());
        inputStream.close();
    }
}
