/*
 * <copyright>
 * 
 * SmpleContentTest.java
 *
 * Licensed Materials - Property of IBM
 * Restricted Materials of IBM
 * 
 * (c) Copyright IBM Corporation 2009.  All Rights Reserved. 
 * Note to U.S. Government Users Restricted Rights:  Use, duplication or  
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp. 
 * 
 * </copyright>
 * 
 */
package com.ibm.sdo.tck.tests.xml;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.util.List;


import junit.framework.Assert;

import org.junit.After;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import com.ibm.sdo.tck.framework.TCKTestBase;
import com.ibm.sdo.tck.util.EqualityTest;
import com.ibm.sdo.tck.util.TCKUtil;
import com.ibm.sdo.tck.util.XMLCompareUtil;

import commonj.sdo.DataObject;
import commonj.sdo.Type;
import commonj.sdo.helper.HelperContext;
import commonj.sdo.helper.XMLDocument;

public class SimpleContentTest extends TCKTestBase 
{
    private HelperContext scope;
	
    @Test
    public void testGetSimpleContentValue() throws Exception 
    {
        URL url = TCKUtil.class.getResource("/SimpleContentTest/simpleContent.xml");
        InputStream inputStream = url.openStream();
        XMLDocument doc = scope.getXMLHelper().load(inputStream);
      //  inputStream.close();
		
        DataObject root = doc.getRootObject();
        Assert.assertEquals("SimpleContent", root.getType().getName());
        Assert.assertEquals("attr_value", root.getString("attribute"));
        Assert.assertEquals("simple_value", root.getString("value"));
    }
	
    
    @Test
    public void testSetSimpleContentValue() throws IOException 
    {
        // Validate serialization of this type
        DataObject dobj = getScope().getDataFactory().create("http://SimpleContent", "SimpleContent");
        dobj.set("value", "simple_value");
        dobj.set("attribute", "attr_value");
        String xml = scope.getXMLHelper().save(dobj, "http://SimpleContent", "simpleContent");
        URL url = TCKUtil.class.getResource("/SimpleContentTest/simpleContent.xml");
        InputStream inputStream = url.openStream();
        assertTrue(XMLCompareUtil.compareXML(inputStream, xml));
        inputStream.close();
    }
    
    @Test
    public void testGetSimpleType() throws Exception 
    {
        URL url = TCKUtil.class.getResource("/SimpleContentTest/booleanContent.xml");
        InputStream inputStream = url.openStream();
        XMLDocument doc = scope.getXMLHelper().load(inputStream);
       // inputStream.close();
                
        DataObject root = doc.getRootObject();
        assertNotNull(root.getInstanceProperty("value"));
        assertEquals(true,root.get("value"));
    }
        
    @Test
    public void testSetSimpleType() throws IOException 
    {
        // Validate serialization of this type
        DataObject dobj = getScope().getDataFactory().create("commonj.sdo", "Boolean");
        dobj.setBoolean("value", true);
        String xml = scope.getXMLHelper().save(dobj, "http://SimpleContent", "booleanContent");
        URL url = TCKUtil.class.getResource("/SimpleContentTest/booleanContent.xml");
        InputStream inputStream = url.openStream();
        assertTrue(XMLCompareUtil.compareXML(inputStream, xml));
        inputStream.close();
    }
    
    @Test
    public void testGetAnyType() throws Exception 
    {
        URL url = TCKUtil.class.getResource("/SimpleContentTest/anyTypeContent.xml");
        InputStream inputStream = url.openStream();
        XMLDocument doc = scope.getXMLHelper().load(inputStream);
      //  inputStream.close();
        
        DataObject root = doc.getRootObject();
        assertEquals("AnyTypeContent", root.getType().getName());
        assertEquals(Boolean.TRUE,root.getDataObject("anyTypeProp").get("value"));
    }
        
    @Test
    public void testSetAnyType() throws IOException 
    {
        // Validate serialization of this type
        DataObject dobj = getScope().getDataFactory().create("http://SimpleContent", "AnyTypeContent");
        DataObject valueDobj = getScope().getDataFactory().create("commonj.sdo", "Boolean");
        valueDobj.setBoolean("value", true);
        dobj.set("anyTypeProp", valueDobj);
        String xml = scope.getXMLHelper().save(dobj, "http://SimpleContent", "anyTypeContent");
        URL url = TCKUtil.class.getResource("/SimpleContentTest/anyTypeContent.xml");
        InputStream inputStream = url.openStream();
        assertTrue(XMLCompareUtil.compareXML(inputStream, xml));
        inputStream.close();
    }
    
    @Test
    // this test validates the spec additions in JSR 235 issue #13
    public void testWrapperObjectsForSimpleTypes()  
    {
        String ns  = "http://SimpleContent";
        
        HelperContext scope = getScope();
        Type anyType = scope.getTypeHelper().getType(ns,"AnyTypeContent");
        assertNotNull(anyType);
        
        DataObject dobjAny = scope.getDataFactory().create(anyType);
        
        Type  charType = scope.getTypeHelper().getType(ns,"char");
        assertNotNull(charType);
        
        DataObject dobj1 = scope.getDataFactory().create(charType);
        assertNotNull(dobj1);
        Type wrapperType1 = dobj1.getType();
        assertFalse(wrapperType1.isDataType());
        dobj1.setString("value", "A");
        assertEquals("A",dobj1.getString("value"));
        
        DataObject dobj2 = scope.getDataFactory().create(ns,"char");
        assertNotNull(dobj2);
        Type wrapperType2 = dobj2.getType();
        assertFalse(wrapperType2.isDataType());
        dobj2.setString("value", "B");
        assertEquals("B",dobj2.getString("value"));
        
        DataObject dobj3 = scope.getDataFactory().create(boolean.class);
        assertNotNull(dobj3);
        Type wrapperType3 = dobj3.getType();
        assertFalse(wrapperType3.isDataType());
        dobj3.setBoolean("value", true);
        assertEquals(true,dobj3.getBoolean("value"));
        
        dobjAny.set("anyTypeProp", dobj1);
        dobjAny.set("anyTypeProp2", dobj2);
        dobjAny.set("anyTypeProp3", dobj3);
        
        String xml = scope.getXMLHelper().save(dobjAny, ns, "anyTypeContent");
        XMLDocument xmldoc = scope.getXMLHelper().load(xml);
        assertNotNull(xmldoc);
        DataObject dobjNew = xmldoc.getRootObject();
        
        EqualityTest et = new EqualityTest(dobjAny, dobjNew, true);
        assertTrue("Serialization and deserialization resulted in a nonequivalent DataObject: " + et.getFailure(),
                   et.getResult() );
    }     
    
    /**
     * When writing tests, it is common to find that several tests need similar
     * objects created before they can run. Annotating a
     * <code>public void</code> method with <code>@Before</code> causes that method to be run before the
     *         {@link org.junit.Test} method. The <code>@Before</code> methods of superclasses will be run before those of the
     *         current class. There is also an <code>@After</code> annotation
     * @throws Exception
     */
    @Override
    @Before
    public void setUp() throws Exception 
    {
        super.setUp();
		
	scope = getScope();
        
        URL url = TCKUtil.class.getResource("/SimpleContentTest/simpleContent.xsd");
        InputStream inputStream = url.openStream();
        List types = scope.getXSDHelper().define(inputStream, url.toString());
        inputStream.close();
        
    }

    @Override
    @After
    public void tearDown() throws Exception 
    {
        super.tearDown();
    }

    /**
     * Sometimes several tests need to share computationally expensive setup
     * (like logging into a database). While this can compromise the
     * independence of tests, sometimes it is a necessary optimization.
     * Annotating a <code>public static void</code> no-arg method with
     * <code>@BeforeClass</code> causes it to be run once before any of the test
     * methods in the class. The <code>@BeforeClass</code> methods of superclasses will be run before those the
     * current class.
     */
    @BeforeClass
    public static void init() 
    {
        // init
    }
}
