/*
 * <copyright>
 * 
 * SubstitutionGroupTest.java
 *
 * Licensed Materials - Property of IBM
 * Restricted Materials of IBM
 * 
 * (c) Copyright IBM Corporation 2009.  All Rights Reserved. 
 * Note to U.S. Government Users Restricted Rights:  Use, duplication or  
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp. 
 * 
 * </copyright>
 * 
 */
package com.ibm.sdo.tck.tests.xml;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

import java.io.IOException;
import java.io.InputStream;
import java.math.BigInteger;
import java.net.URL;
import java.util.List;


import org.junit.After;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import com.ibm.sdo.tck.framework.TCKTestBase;
import com.ibm.sdo.tck.util.TCKUtil;
import com.ibm.sdo.tck.util.XMLCompareUtil;

import commonj.sdo.DataObject;
import commonj.sdo.Property;
import commonj.sdo.Type;
import commonj.sdo.helper.DataFactory;
import commonj.sdo.helper.XMLDocument;
import commonj.sdo.helper.XMLHelper;
import commonj.sdo.helper.XSDHelper;

public class SubstitutionGroupTest extends TCKTestBase 
{
    private XSDHelper xsdHelper;
    private XMLHelper xmlHelper;

    /**
     * Test to create a substitutable concrete element/DataObject by calling
     * DataObject.createDataObject(Property) method
     * 
     * @throws IOException
     */
    @Test
    public void substitutionGroupTest1() throws IOException 
    {
        URL url = TCKUtil.class.getResource("/SubstitutionGroupTest/person.xml");
        InputStream inputStream = url.openStream();
        XMLDocument xmldoc = xmlHelper.load(inputStream);
        inputStream.close();  
        
        assertNotNull(xmldoc);
        DataObject dobj = xmldoc.getRootObject();
        Property personProp = dobj.getType().getProperty("person");
        assertNotNull(personProp);
        List persons = dobj.getList(personProp);
        assertEquals(3,persons.size());
        String [] names = {"John","Mary","Steve" };
        for( int i=0; i<persons.size(); i++ )
        {
            DataObject person = (DataObject)persons.get(i);
            String name = person.getString("name");
            assertEquals(names[i],name);
        }
    }

    /**
     * When writing tests, it is common to find that several tests need similar
     * objects created before they can run. Annotating a
     * <code>public void</code> method with <code>@Before</code> causes that method to be run before the
     *         {@link org.junit.Test} method. The <code>@Before</code> methods of superclasses will be run before those of the
     *         current class. There is also an <code>@After</code> annotation
     * @throws Exception
     */
    @Override
    @Before
    public void setUp() throws Exception 
    {
        super.setUp();
		
        // Set up class instance variable
	xsdHelper = getScope().getXSDHelper();
	xmlHelper = getScope().getXMLHelper();
		
        URL url = TCKUtil.class.getResource("/SubstitutionGroupTest/person.xsd");
        InputStream inputStream = url.openStream();
        xsdHelper.define(inputStream, url.toString());
        inputStream.close();  
    }
}
