/*
 * <copyright>
 * 
 * EqualityTest.java
 *
 * Licensed Materials - Property of IBM
 * Restricted Materials of IBM
 * 
 * (c) Copyright IBM Corporation 2009.  All Rights Reserved. 
 * Note to U.S. Government Users Restricted Rights:  Use, duplication or  
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp. 
 * 
 * </copyright>
 * 
 */
package com.ibm.sdo.tck.util;

import java.math.BigDecimal;
import java.math.BigInteger;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;
import java.util.Date;
import java.util.Iterator;
import java.util.List;

import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.XMLGregorianCalendar;

import commonj.sdo.DataObject;
import commonj.sdo.Property;
import commonj.sdo.Sequence;
import commonj.sdo.Type;
import commonj.sdo.helper.HelperContext;
import commonj.sdo.helper.TypeHelper;
import commonj.sdo.impl.HelperProvider;

public class EqualityTest 
{
    public static class GeneralComparator implements Comparator 
    {
        /**
         * The compare method for the GeneralComparator class is used to compare
         * two of any Types between which a conversion is permitted in SDO.
         */
    
        public int compare(Object obj1, Object obj2) {
            if (obj1.getClass() == obj2.getClass()) {
                if( obj1 instanceof byte [] ) {
                    if( ((byte [])obj1).length == ((byte [])obj2).length ) {
                        int retval = 0;
                        byte [] byteArray1 = (byte [])obj1;
                        byte [] byteArray2 = (byte [])obj2;
                        for( int i=0; i<byteArray1.length; i++ ) {
                            if( byteArray1[i] != byteArray2[i] ) {
                                retval = 1;
                                break;
                            }
                        }                    
                        return retval;
                    }
                    else
                        return 1;
                }
                if( obj1 instanceof BigDecimal )
                    return(((BigDecimal)obj1).compareTo((BigDecimal)obj2));
                else {
                    if (obj1.equals(obj2))
                        return 0;
                    else
                        return 1;
                }
            }
    
            else if (obj1.getClass() == Date.class) {
                if (obj2.getClass() == String.class) {
                    try {
                        XMLGregorianCalendar c2 = DatatypeFactory.newInstance().newXMLGregorianCalendar((String)obj2);
                        obj2 = c2.toGregorianCalendar().getTime();
                        if (obj1.equals(obj2))
                            return 0;
                    } catch (Exception e) {
                        System.out.println(e.getMessage());
                    }
    
                    return 1;
                }
    
                else {
                    Date temp = (Date)obj1;
                    Long val = new Long(temp.getTime());
                    return compare(val, obj2);
                }
    
            }
    
            else if (obj2.getClass() == Date.class) {
                return compare(obj2, obj1);
            }
    
            else if (obj1.getClass() == Boolean.class) {
                Boolean temp = (Boolean)obj1;
    
                if (temp.booleanValue()) {
                    if (obj2.toString().equalsIgnoreCase("true"))
                        return 0;
                    else
                        return 1;
                }
    
                else {
                    if (obj2.toString().equalsIgnoreCase("true"))
                        return 1;
                    else
                        return 0;
                }
            }
    
            else if (obj2.getClass() == Boolean.class)
                return compare(obj2, obj1);
    
            else if (obj1.getClass() == Byte.class || obj2.getClass() == Byte.class) {
                byte b1 = (Double.valueOf(obj1.toString())).byteValue();
                byte b2 = (Double.valueOf(obj2.toString())).byteValue();
    
                if (b1 == b2)
                    return 0;
                else if (b1 < b2)
                    return -1;
                else
                    return 1;
            }
    
            else if (obj1 instanceof byte []) {
                if( obj2.getClass() == String.class )
                {
                    // This test handles the unique byte [] to String 
                    // Conversion specified by SDO
                    String testStr = (String)obj2;
                    byte[] compBytes = new byte[testStr.length()/2];
                    if( testStr.length()/2 >= 1 )
                    {
                        for( int i=0; i<testStr.length(); i+=2 )
                        {
                            byte c = (byte)Integer.parseInt(testStr.substring(i, i+2),16);
                            compBytes[i/2] = c;
                        }
                    }
                    if( java.util.Arrays.equals((byte[])obj1,compBytes) )
                        return 0;
                    else
                        return 1;
                }
                else
                {
                    long result = 0;
                    long multiplier = 1;
    
                    byte[] array = (byte[])obj1;
                    for (int i = 0; i < array.length; i++) {
                        result += array[array.length - i - 1] * multiplier;
                        multiplier *= 256;
                    }
                    return compare(obj2, new Long(result));
                }        
            }
    
            else if (obj2 instanceof byte []) {
                return compare(obj2, obj1);
            }
    
            else if (obj1.getClass() == Short.class || obj2.getClass() == Short.class) {
                short s1 = (Double.valueOf(obj1.toString())).shortValue();
                short s2 = (Double.valueOf(obj2.toString())).shortValue();
    
                if (s1 == s2)
                    return 0;
                else if (s1 < s2)
                    return -1;
                else
                    return 1;
            }
    
            else if (obj1.getClass() == Integer.class || obj2.getClass() == Integer.class) {
                int i1 = (Double.valueOf(obj1.toString())).intValue();
                int i2 = (Double.valueOf(obj2.toString())).intValue();
    
                if (i1 == i2)
                    return 0;
                else if (i1 < i2)
                    return -1;
                else
                    return 1;
            }
    
            else if (obj1.getClass() == Long.class || obj2.getClass() == Long.class
                || obj1.getClass() == BigInteger.class
                || obj2.getClass() == BigInteger.class) {
                long l1 = (Double.valueOf(obj1.toString())).longValue();
                long l2 = (Double.valueOf(obj2.toString())).longValue();
    
                if (l1 == l2)
                    return 0;
                else if (l1 < l2)
                    return -1;
                else
                    return 1;
            }
    
            else if (obj1.getClass() == Float.class || obj2.getClass() == Float.class) {
                float f1 = (Double.valueOf(obj1.toString())).floatValue();
                float f2 = (Double.valueOf(obj2.toString())).floatValue();
    
                if (f1 == f2)
                    return 0;
                else if (f1 < f2)
                    return -1;
                else
                    return 1;
            }
    
            else if (obj1.getClass() == Double.class || obj2.getClass() == Double.class) {
                Double b1 = Double.valueOf(obj1.toString());
                Double b2 = Double.valueOf(obj2.toString());
    
                return b1.compareTo(b2);
            }
    
            else if (obj1.getClass() == BigDecimal.class || obj2.getClass() == BigDecimal.class) {
                BigDecimal b1 = new BigDecimal(obj1.toString());
                BigDecimal b2 = new BigDecimal(obj2.toString());
    
                return b1.compareTo(b2);
            }
    
            else {
                if (obj1.toString().equals(obj2.toString()))
                    return 0;
                else
                    return 1;
            }
        }
    }

    private boolean result = true;
    private String failure = "";
    
    public EqualityTest(DataObject dobj1, DataObject dobj2, boolean deep)
    {
        performCompare(dobj1,dobj2,deep);
    }

    public boolean getResult() {return result;}
    public String getFailure() {return failure;}
    
    private void performCompare(DataObject dobj1, DataObject dobj2, boolean deep )
    {
        Type t1 = dobj1.getType();
        Type t2 = dobj2.getType();
        
        if( !t1.equals(t2) ) 
        {
            result  = false;
            failure = "Types are not equivalent.";
            return;
        }    

        List l1 = dobj1.getInstanceProperties();
        List l2 = dobj2.getInstanceProperties();
        
        if( l1.size() != l2.size() )
        {
            result  = false;
            failure = "The sizes of instance properties are not equivalent.";
            return;
        }    
        
        Iterator it = l1.iterator();
        while(it.hasNext()) 
        {
            Property p1 = (Property)it.next();
            if( dobj1.isSet(p1) )
            {
                List vals1, vals2;

                vals1 = new ArrayList();
                vals2 = new ArrayList();
                
                if( p1.isMany() )
                {
                    vals1 = (List)dobj1.get(p1);
                    vals2 = (List)dobj2.get(p1);
                    
                    if( ( p1.getType().isDataType() || deep) &&
                        vals1.size() != vals2.size() )
                    {    
                        result  = false;
                        failure = "The sizes of many valued property '" + p1.getName() + "' are not equivalent [" + vals1.size() +","+vals2.size()+"].";
                        return;
                    }
                    
                }
                else
                {
                    Type changeSummaryType = TypeHelper.INSTANCE.getType("commonj.sdo","ChangeSummaryType");
                    if( p1.getType() != changeSummaryType )
                    {
                        // don't compare change summary properties
                        vals1.add(dobj1.get(p1));
                        vals2.add(dobj2.get(p1));
                    }
                }

                for( int i=0; i<vals1.size(); i++ )
                {
                    if( p1.getType().isDataType() )
                    {
                        if( "Bytes".equals(p1.getType().getName()) )
                        {
                            byte [] v1 = (byte [])vals1.get(i);
                            byte [] v2 = (byte [])vals2.get(i);
                            
                            if( v1 == null || v2 == null )
                            {
                                if( v1 != v2 )
                                {
                                    result  = false;
                                    failure = "Bytes values for property '" + p1.getName() + "' are not equivalent. One of the arrays is null.";
                                    return;
                                }
                            }
                            else
                            {
                                if( v1.length != v2.length )
                                {    
                                    result  = false;
                                    failure = "Bytes values for property '" + p1.getName() + "' are not equivalent. Bytes arrays are different lengths.";
                                    return;
                                }
                            
                                for( int ii=0; ii<v1.length; ii++ )
                                {
                                    if( v1[ii] != v2[ii] )
                                    {    
                                        result  = false;
                                        failure = "Bytes values for property '" + p1.getName() + "' are not equivalent. [" + v1[ii] + "," + v2[ii] + "]" ;
                                        return;
                                    }
                                }
                            }
                        }
                        else
                        {
                            if( vals1.get(i) == null )
                            {
                                if( vals2.get(i) != null )
                                {
                                    result  = false;
                                    failure = "The values for property '" + p1.getName() + "' are not equivalent.  [" + vals1.get(i) + "," + vals2.get(i) + "]";
                                    return;
                                }
                                
                            }
                            else
                            {
                                if(!compareSimpleTypes(vals1.get(i),vals2.get(i)))
                                {    
                                    result  = false;
                                    failure = "The values for property '" + p1.getName() + "' are not equivalent.   [" + vals1.get(i) + "," + vals2.get(i) + "]";
                                    return;
                                }
                            }
                        }    
                    }
                    else
                    {
                        if( deep )
                        {
                            EqualityTest deepCompare = new EqualityTest((DataObject)vals1.get(i),
                                                                        (DataObject)vals2.get(i),
                                                                        p1.isContainment());
                            if( !deepCompare.getResult() )
                            {    
                                result  = false;
                                failure = "Deep copy compare failed for property '" + p1.getName() + "'.\r\n";
                                failure += "Failure cause was '" + deepCompare.getFailure() + "'.";
                                
                                return;
                            }    
                        }
                    }
                }
            }    
        }
        
        if( t1.isSequenced() )
        {
            Sequence s1 = dobj1.getSequence();
            Sequence s2 = dobj2.getSequence();
            
            if( s1.size() != s2.size() )
            {    
                result  = false;
                failure = "The sizes of the sequences are not equivalent.";
                return;
            }    

            for( int i=0; i<s1.size(); i++ )
            {
                if( s1.getProperty(i) != s2.getProperty(i) )
                {    
                    result  = false;
                    failure = "The properties in sequence position (" + i + ") are not equivalent.";
                    return;
                }    
                
                if( s1.getProperty(i) == null )
                {
                    if( !s1.getValue(i).equals(s2.getValue(i)) )
                    {    
                        result  = false;
                        failure = "The values in sequence position (" + i + ") are not equivalent.";
                        return;
                    }    
                }
            }
        }
    }
    
    private boolean compareSimpleTypes(Object val1, Object val2)
    {
        boolean retval = true;
        
        // for most cases we can default to the Object.equals method, but
        // for decimal point values, we need to be more selective in comparing
        if( val1 instanceof BigDecimal )
        {
            int ret = ((BigDecimal)val1).compareTo((BigDecimal)val2);
            if( ret != 0 )
                retval = false;
        }
        else
            retval = val1.equals(val2);
        
        return retval;
    }
}
