/*
 *  Licensed to the Apache Software Foundation (ASF) under one
 *  or more contributor license agreements.  See the NOTICE file
 *  distributed with this work for additional information
 *  regarding copyright ownership.  The ASF licenses this file
 *  to you under the Apache License, Version 2.0 (the
 *  "License"); you may not use this file except in compliance
 *  with the License.  You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an
 *  "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 *  KIND, either express or implied.  See the License for the
 *  specific language governing permissions and limitations
 *  under the License.
 *  
 *  $Rev: 538980 $  $Date: 2008/07/28 19:50:46 $
 */
package commonj.sdo.impl;

import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.OutputStream;

import commonj.sdo.DataGraph;
import commonj.sdo.helper.HelperContext;

/**
 * Vendors can provide an implementation of this interface to bootstrap their
 * SDO implementation.
 */
public interface RuntimeHelper {
    /**
     * Create an empty data graph.
     * 
     * @return the new data graph instance.
     */
    public DataGraph createDataGraph(HelperContext hc);

    /**
     * Create a new HelperContext, a new scope
     * 
     * @return the new HelperContext instance.
     */
    public HelperContext createHelperContext();
    
    /**
     * Create a new ObjectOutputStream
     * 
     * @return the new ObjectOutputStream instance.
     */
    public ObjectOutputStream createObjectOutputStream(OutputStream os, HelperContext helperContext);

    /**
     * Create a new ObjectInutStream
     * 
     * @return the new ObjectInputStream instance.
     */
    public ObjectInputStream createObjectInputStream(InputStream is, HelperContext helperContext);
    
    /**
     * Make Static Types available to the runtime within the context of "scope"
     * 
     * @param scope the scope for the new Types
     * @param namespace the namespace of the type set to define
     * @param location an indicator for where the types may be found
     */
    public HelperContext defineStaticTypes(HelperContext scope, String namespace, String location, ClassLoader currentCL);
    
}
